// EAllocatorObj.h: interface for the EAllocatorObj class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EALLOCATOROBJ_H__CB0A9031_BC16_11D2_91F2_004005446EE6__INCLUDED_)
#define AFX_EALLOCATOROBJ_H__CB0A9031_BC16_11D2_91F2_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// An equivalent definition of new is already presents (STL...).
#ifndef ETOOLLIB_NO_NEW_ALLOCATOR

/*! define a global new allocator for object creation.
 * @param nSize Size of the object to allocate
 * @param pPtr Parameter, adress of the allocated object (in).
 * @return Pointer on the allocated object (same as pPtr).
 */
inline void *
operator new( size_t nSize, 
              void *pPtr )
{
  return pPtr; 
}

#if _MSC_VER >= 1200
// removes the warning "no matching delete operator..." with VC 6.
inline void 
operator delete( void *pObject, 
                 void *pPtr )
{
  return; 
}
#endif

#endif


#include "EAllocator.h"

template< class TElm,class EParentAllocator = EAllocatorT< TElm > >
class EAllocatorObjT : public EParentAllocator
{
public:
  EAllocatorObjT() {}
  virtual ~EAllocatorObjT() {}

  /*! Initializes an object of the array.
   *
   * @param nIndex Zero based index of the object to initialize.
   */
  virtual void construct( SL nIndex );

  /*! Initializes a sequence of object in the array.
   *
   * @param nIndex Zero based index of the first object to initialize.
   * @param nLen Number of object to initialize.
   */
  virtual void construct( SL nIndex,
                          SL nLen );

  /*! Destructs an object of the array.
   * @param nIndex Zero based index of the object to destruct.
   */
  virtual void destruct( SL nIndex );

  /*! Destructs a sequence list of object in the array.
   * @param nIndex Zero based index of the first object to destruct. 
   * @param nLen Number of object to destruct. May be 0.
   */
  virtual void destruct( SL nIndex, 
                         SL nLen );

  /*! Copy a sequence of object from this array to another one.
   * @param nIndexSource Zero based index of the first object of the sequence to copy.
   * @param nLen Number of object in the sequence. May be zero.
   * @param nIndexDestination Zero based index of destination array where the sequence
   *                          is copied to.
   * @param allocatorDest Destination array.
   */
  virtual void copyTo( SL nIndexSource, 
                       SL nLen,
                       SL nIndexDestination,
                       EAllocatorT<TElm> &allocatorDest ) const;

  /*! Copy a sequence of object from a standard C array to this array.
   * @param arraySource Pointer on the first element of the source array to copy.
   * @param nLen Number of objects in the sequence. May be zero.
   * @param nIndexDestination Zero based index where the sequence is copied to.
   */
  virtual void copyFrom( const TElm *arraySource, 
                         SL nLen,
                         SL nIndexDestination );

  /*! Move a sequence of object inside the array.
   * @param nIndex Zero based index of the first element to move.
   * @param nLen Number of elements to move.
   * @param nIndexDestination Zero based index where the first element is moved to.
   */
  virtual void copyToThis( SL nIndex,
                           SL nLen,
                           SL nIndexDestination );
  /**@shapeType DependencyLink*/
  /*#  EAllocatorT lnkUnnamed*/
};



template<class TElm,class EParentAllocator>
void 
EAllocatorObjT<TElm,EParentAllocator>::copyTo( SL nIndexSource, 
                                               SL nLen,
                                               SL nIndexDestination,
                                               EAllocatorT<TElm> &allocatorDest ) const
{
  EASSERT( nIndexSource <= count() );
  EASSERT( nLen >= 0 );
  EASSERT( nIndexSource + nLen <= count() );
  EASSERT( nIndexDestination <= allocatorDest.count() );
  EASSERT( nIndexDestination + nLen <= allocatorDest.count() );

  if ( !nLen )
    return;


  TElm *pDstElm = &allocatorDest.at( nIndexDestination );
  const TElm *pSrcElm = &at( nIndexSource );

  while ( nLen-- )
  {
    *pDstElm++ = *pSrcElm++;
  }
}


template<class TElm,class EParentAllocator>
void 
EAllocatorObjT<TElm,EParentAllocator>::copyFrom( const TElm *arraySource, 
                                                 SL nLen,
                                                 SL nIndexDestination )
{
  EASSERT( nIndexDestination >= 0 );
  EASSERT( nLen >= 0 );
  EASSERT( nIndexDestination + nLen <= count() );

  if ( !nLen )
    return;

  EASSERT( arraySource != NULL );

  TElm *pDstElm = &at( nIndexDestination );
  while ( nLen-- )
  {
    *pDstElm++ = *arraySource++;
  }
}


template<class TElm,class EParentAllocator>
void 
EAllocatorObjT<TElm,EParentAllocator>::copyToThis( SL nIndex,
                                                   SL nLen,
                                                   SL nIndexDestination )
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex + nLen <= count() );
  EASSERT( nIndexDestination >= 0 );
  EASSERT( nIndexDestination + nLen <= count() );

  if ( nIndex == nIndexDestination  ||
       !nLen )
    return;

  if ( nIndex > nIndexDestination )
  {
    TElm *pDstElm = &at( nIndexDestination );
    const TElm *pSrcElm = &at( nIndex );
    while ( nLen-- )
      *pDstElm++ = *pSrcElm++;
  }
  else      // reverse copy so we do not overwrite source will copying.
  {
    TElm *pDstElm = &at( nIndexDestination ) + nLen;
    const TElm *pSrcElm = &at( nIndex ) + nLen;
    while ( nLen-- )
      *--pDstElm = *--pSrcElm;
  }
}


template<class TElm,class EParentAllocator>
inline void 
EAllocatorObjT<TElm,EParentAllocator>::construct( SL nIndex )
{
  new ((void *)&at( nIndex )) TElm;
}


template<class TElm,class EParentAllocator>
inline void 
EAllocatorObjT<TElm,EParentAllocator>::construct( SL nIndex,
                                                  SL nLen )
{
  EASSERT( nLen >= 0 );
  EASSERT( nIndex + nLen <= count() );
  while ( nLen-- )
    new (&at( nIndex++ )) TElm;
}


template<class TElm,class EParentAllocator>
void 
EAllocatorObjT<TElm,EParentAllocator>::destruct( SL nIndex )
{
  at( nIndex ).~TElm();
}


template<class TElm,class EParentAllocator>
void 
EAllocatorObjT<TElm,EParentAllocator>::destruct( SL nIndex, 
                                                 SL nLen )
{
  EASSERT( nLen >= 0 );
  EASSERT( nIndex + nLen <= count() );
  while ( nLen-- )
    at( nIndex++ ).~TElm();
}

#endif // !defined(AFX_EALLOCATOROBJ_H__CB0A9031_BC16_11D2_91F2_004005446EE6__INCLUDED_)
