// EAutoPtrT.h: interface for the EAutoPtrT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EAUTOPTRT_H__49E23C52_834A_11D3_BEC1_0040053AD648__INCLUDED_)
#define AFX_EAUTOPTRT_H__49E23C52_834A_11D3_BEC1_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

template<class Type>
class EAutoPtrT
{
public:
  typedef EAutoPtrT<Type> ThisType;

  /** Construct a auto pointer.
   */
	EAutoPtrT( Type *pObject =NULL );

  /*! Takes the ownership of a pointer from another auto pointer.
   * @param ptrToTake Auto pointer which this object take the ownership.
   */
  EAutorPtrT( ThisType &ptrToTake );

  /** Destructor.
   *
   * If the object is attached, it is destroyed.
   */
	virtual ~EAutoPtrT();

  /** Detach the object.
   * @return Pointer on the object that was attached. May be \c NULL.
   */
  Type *detach();

  Type *operator->() const;

  operator Type *() const;

  operator bool() const;
  bool operator !() const;

  /*! Take the ownership of a pointer.
   * @param pObject Pointer that this auto pointer take the ownership.
   * @return Reference on this object.
   */
  ThisType &operator =( Type *pObject );

  /*! Take the ownership of another auto pointer.
   * @param ptrToTake Auto pointer which this object take the ownership.
   * @return Reference on this object.
   */
  ThisType &operator =( ThisType &ptrToTake );


private:
  /** Pointer on the object. May be \c NULL.
   */
  Type *_pObject;

  /*! Indicates if the pointer is owned.
   */
  bool _bIsOwned;
};



template<class Type>
inline
EAutoPtrT<Type>::EAutoPtrT( Type *pObject ) :
    _pObject( pObject ),
    _bIsOwned( true )
{
}


template<class Type>
inline
EAutoPtrT<Type>::EAutorPtrT( ThisType &ptrToTake ) :
    _pObject( ptrToTake._pObject ),
    _bIsOwned( true )
{
  ptrTotake._bIsOwned = false;
  ptrToTake._pObject = NULL;
}


template<class Type>
inline
EAutoPtrT<Type>::~EAutoPtrT()
{
  if ( _bIsOwned  &&  _pObject )
    delete _pObject;
}


template<class Type>
inline Type *
EAutoPtrT<Type>::detach()
{
  Type *pObject = _pObject;
  _pObject = NULL;
  _bIsOwned = false;
  return pObject;
}


template<class Type>
inline Type *
EAutoPtrT<Type>::operator->() const
{
  EASSERT( _pObject );
  return _pObject;
}


template<class Type>
inline
EAutoPtrT<Type>::operator Type *() const
{
  return _pObject;
}


template<class Type>
inline
EAutoPtrT<Type>::operator bool() const
{
  return _pObject != NULL;
}


template<class Type>
inline bool
EAutoPtrT<Type>::operator !() const
{
  return _pObject == NULL;
}


template<class Type>
inline EAutoPtrT<Type> &
EAutoPtrT<Type>::operator =( Type *pObject )
{
  if ( _bIsOwned  &&  _pObject )
    delete _pObject;
  _pObject = pObject;
  _bIsOwned = true;
  return *this;
}


template<class Type>
inline EAutoPtrT<Type> &
EAutoPtrT<Type>::operator =( ThisType &ptrToTake )
{
  if ( _bIsOwned  &&  _pObject )
    delete _pObject;
  _pObject = ptrToTake._pObject;
  _bIsOwned = _ptrToTake._bIsOwned;
  _ptrToTake._bIsOwned = false;
  return *this;
}


#endif // !defined(AFX_EAUTOPTRT_H__49E23C52_834A_11D3_BEC1_0040053AD648__INCLUDED_)
