// EBaseClass.h: interface for the EBaseClass class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASECLASS_H__25480D14_4886_11D2_AE44_E6FF6EE0B361__INCLUDED_)
#define AFX_EBASECLASS_H__25480D14_4886_11D2_AE44_E6FF6EE0B361__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EDataStr.hpp"
#include "EObjectContext.h"
class EClassId;

/*! This class is the base class for any class that support identification.
 */
class EBaseClass  
{
public:
  EBaseClass() {};
  virtual ~EBaseClass() {};

  /* Get the EClassId object of the most derivated class.
   * @return EClassId object of the lowest class in the inheritance tree.
   */
  virtual const EClassId &objectId() const =0;


  /*! Prepares object context for saving.
   * @param ctx Object context to prepare.
   */
  virtual void prepareObjectContext( EObjectContext &ctx ) const;

  /*! Saves persistant object.
   *
   * Save class name & version, then call saveObjectData().
   */
  EDataStream &saveObject( EDataStream &s ) const;

  /*! Loads persistant object.
   * Loads class name & version, creates object and call loadObjectData().
   * @param s Datastream.
   * @param pObject on return, created & loaded object.
   */
  static EDataStream &loadObject( EDataStream &s, 
                                  EBaseClass **pObject );

  /*! Saves object data.
   *
   * Default does nothing.
   */
  virtual EDataStream &saveObjectData( EDataStream &s,
                                       const EObjectContext &ctx ) const;

  /*! Loads object data.
   *
   * Default does nothing.
   */
  virtual EDataStream &loadObjectData( EDataStream &s,
                                       const EObjectContext &ctx );
};

#endif // !defined(AFX_EBASECLASS_H__25480D14_4886_11D2_AE44_E6FF6EE0B361__INCLUDED_)
