// EBaseHashIteratorT.h: interface for the EBaseHashIteratorT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASEHASHITERATORT_H__9BDBE5A3_E110_11D2_9222_004005446EE6__INCLUDED_)
#define AFX_EBASEHASHITERATORT_H__9BDBE5A3_E110_11D2_9222_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "EIteratorT.h"

template<class Key, class Item, class KeyArg, class ItemArg>
class EBaseHashIteratorT : public EIteratorT<Item>
{
public:
  EBaseHashIteratorT( const EBaseHashT<Key,Item,KeyArg,ItemArg> &hashTable );
  EBaseHashIteratorT();

  void attach(  const EBaseHashT<Key,Item,KeyArg,ItemArg> &hashTable );

  /** Set the iterator at the beginning of the table.
   * If there is no element in the table then isEnd() will return true.
   */
  void begin();

  /** Test if the iterator is beyond the last association.
   */
  bool isEnd() const;

  /** Remove the association at the current position of the iterator.
   */
  void remove();

  /** Move the iterator to the next association.
   */
  void next();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  Item& item();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  const Item& item() const;

  /** Get the key at the current position of the iterator.
   * @return key at the current position of the iterator.
   */
  const Key& key() const;

  /*! Get the numbers of items.
   * @return Number of items.
   */
  SL count() const;

private:
  /** Search the next index with an association.
   * The search begins at the index _nIndex +1.
   */
  void searchNextIndex();

  /// Hash table the iterator operate on.
  const EBaseHashT<Key,Item,KeyArg,ItemArg> *_pHashTable;

  /// Current index in association table.
  SL _nIndex;

  /// Current Association for the list at nIndex.
  const EBaseHashT<Key,Item,KeyArg,ItemArg>::ELCAsso *_pAsso;
};



// Template implementation:
// #####################Last change: 28/03/98 20h01mn#########################

template<class Key, class Item, class KeyArg, class ItemArg>
EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::EBaseHashIteratorT(
    const EBaseHashT<Key,Item,KeyArg,ItemArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}


template<class Key, class Item, class KeyArg, class ItemArg>
EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::EBaseHashIteratorT()
{
  _pHashTable = NULL;
  _nIndex = 0;
  _pAsso = ENIL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void
EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::attach(
    const EBaseHashT<Key,Item,KeyArg,ItemArg> &hashTable )
{
  EASSERT( pHashTable );
  _pHashTable = pHashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}

template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::begin()
{
  EASSERT( _pHashTable );
  _nIndex = -1;
  searchNextIndex();
}

template<class Key, class Item, class KeyArg, class ItemArg>
bool EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::isEnd() const
{
  EASSERT( _pHashTable );
  return _pAsso == ENIL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::remove()
{
  EASSERT( _pHashTable );
  if ( isEnd() )
    return;

  const EBaseHashT<Key,Item,KeyArg,ItemArg>::ELCAsso *pAsso =
      _pHashTable->_aAsso[ _nIndex ];
  const EBaseHashT<Key,Item,KeyArg,ItemArg>::ELCAsso **pPrevAsso =
      &_pHashTable->_aAsso[ _nIndex ];

  while ( pAsso != _pAsso )     // search previous association
  {
    EASSERT( pAsso );
    pPrevAsso = &pAsso->_pNext;
    pAsso = pAsso->_pNext;
  }

  _pHashTable->doFreeAsso( _pAsso->_key, _pAsso->_item );

  *pPrevAsso = _pAsso->_pNext;       // unchain
  delete _pAsso;
  _pAsso = *pPrevAsso;
  _pHashTable->_nNbAsso--;

  if ( !_pAsso )                  // if no more in the list for this index
  {                               // then search next.
    searchNextIndex();
  }
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::next()
{
  EASSERT( _pHashTable );
  if ( _pAsso )
  {
    _pAsso = _pAsso->_pNext;
    if ( !_pAsso )          // end of current index reached ?
      searchNextIndex();
  }
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::searchNextIndex()
{
  EASSERT( _pHashTable );
  SL nSize = _pHashTable->size();
  _nIndex++;
  for ( ;_nIndex < nSize; _nIndex++ )
  {
    if ( _pHashTable->_aAsso[ _nIndex ] )
    {
      _pAsso = _pHashTable->_aAsso[ _nIndex ];
      return;
    }
  }
  _pAsso = ENIL;
}

template<class Key, class Item, class KeyArg, class ItemArg>
const Key& EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::key() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_key;
}

template<class Key, class Item, class KeyArg, class ItemArg>
const Item& EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::item() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_item;
}

template<class Key, class Item, class KeyArg, class ItemArg>
Item& EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::item()
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return *(Item *)&_pAsso->_item;
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL 
EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>::count() const
{
  EASSERT( _pHashTable );
  return _pHashTable->count();
}


#endif // !defined(AFX_EBASEHASHITERATORT_H__9BDBE5A3_E110_11D2_9222_004005446EE6__INCLUDED_)
