// ###########################################################################
// File: ECallback2T.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK2T_H__B78AA25A_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK2T_H__B78AA25A_D476_11D2_BD55_0040053AD648__INCLUDED_

/*
// ECallback2T

#ECallback=ECallback2T
#PARAM_CALL=TP1 p1,TP2 p2
#PARAM_CALL_SEP=, TP1 p1,TP2 p2
#PARAM_DECL=TP1,TP2
#PARAM_DECL_FN=TP1Fn,TP2Fn
#CALL_LIST=p1,p2
#CALL_LIST_SEP=, p1,p2
#SUFFIX=2T
#TEMPLATE_PARAM_RET=class TP1,class TP2
#TEMPLATE_PARAM=,class TP1,class TP2
#TEMPLATE_PARAM_RET_SEP=,class TP1,class TP2
#TEMPLATE_PARAM_RET_SEP_ARG=,TP1,TP2
#TEMPLATE_PARAM_CB=<class TP1,class TP2>
#TEMPLATE_PARAM_CB_ARG=<TP1,TP2>
#TEMPLATE_CB_DECL=template<class TP1,class TP2>
#TEMPLATE_PARAM_FN=,class TP1Fn,class TP2Fn
#RETURN_FN=ReturnType
#RET=void
#RETURN=
*/


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2>
class ECallback2T : protected EBaseCallback
{
public:
  ECallback2T() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()(TP1 p1,TP2 p2) const;


  typedef void (*ECallerFn)(const EBaseCallback &cb, TP1 p1,TP2 p2);

  ECallback2T( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2 >
class ECallbackCallerFn2T : public ECallback2T<TP1,TP2>
{
public:
  ECallbackCallerFn2T( Function fnCallback );

  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2
         ,class TP1Fn,class TP2Fn>
inline ECallbackCallerFn2T< ReturnType (*)(TP1Fn,TP2Fn)
                                   ,TP1,TP2 >
eCbCreator( ECallback2T<TP1,TP2> *, 
                         ReturnType (*fn)(TP1Fn,TP2Fn) )
{
  return ECallbackCallerFn2T< ReturnType (*)(TP1Fn,TP2Fn)
                            ,TP1,TP2>( fn );
}

template< class Receiver, class Member,class TP1,class TP2>
class ECallbackCallerMember2T : public ECallback2T<TP1,TP2>
{
public:
  ECallbackCallerMember2T( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2 );
};


template< class Receiver, class CallObjType
,class TP1Fn,class TP2Fn
,class TP1,class TP2,class ReturnType >
inline ECallbackCallerMember2T< Receiver, 
                                       ReturnType (CallObjType::*)(TP1Fn,TP2Fn)
                                       ,TP1,TP2 >
eCbCreator( ECallback2T<TP1,TP2> *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn) )
{
  typedef ReturnType (CallObjType::*MemberCallback)(TP1Fn,TP2Fn);

  return ECallbackCallerMember2T< Receiver, MemberCallback
                                ,TP1,TP2>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2>
inline 
ECallback2T<TP1,TP2>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2>
inline 
void
ECallback2T<TP1,TP2>::operator ()(TP1 p1,TP2 p2) const
{
   _fnCaller( *this, p1,p2 );
}


template<class TP1,class TP2>
inline
ECallback2T<TP1,TP2>::ECallback2T( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2 >
inline
ECallbackCallerFn2T< Function,TP1,TP2 >::ECallbackCallerFn2T( Function fnCallback ) :
    ECallback2T<TP1,TP2>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2 >
inline void
ECallbackCallerFn2T< Function,TP1,TP2 >::caller( const EBaseCallback &cb, TP1 p1,TP2 p2 )
{
   ((Function)cb._fnCallback)(p1,p2);
}


template< class Receiver, class Member,class TP1,class TP2 >
inline 
ECallbackCallerMember2T<Receiver,Member,TP1,TP2>::ECallbackCallerMember2T( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback2T<TP1,TP2>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2 >
inline void
ECallbackCallerMember2T<Receiver,Member,TP1,TP2>::caller( const EBaseCallback &cb, TP1 p1,TP2 p2 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)(p1,p2);
}

#endif // !defined(AFX_ECALLBACK2T_H__B78AA25A_D476_11D2_BD55_0040053AD648__INCLUDED_)
