// ###########################################################################
// File: ECallback3T.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK3T_H__B78AA25C_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK3T_H__B78AA25C_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3>
class ECallback3T : protected EBaseCallback
{
public:
  ECallback3T() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()(TP1 p1,TP2 p2,TP3 p3) const;


  typedef void (*ECallerFn)(const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3);

  ECallback3T( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3 >
class ECallbackCallerFn3T : public ECallback3T<TP1,TP2,TP3>
{
public:
  ECallbackCallerFn3T( Function fnCallback );

  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3
         ,class TP1Fn,class TP2Fn,class TP3Fn>
inline ECallbackCallerFn3T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn)
                                   ,TP1,TP2,TP3 >
eCbCreator( ECallback3T<TP1,TP2,TP3> *, 
                         ReturnType (*fn)(TP1Fn,TP2Fn,TP3Fn) )
{
  return ECallbackCallerFn3T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn)
                            ,TP1,TP2,TP3>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3>
class ECallbackCallerMember3T : public ECallback3T<TP1,TP2,TP3>
{
public:
  ECallbackCallerMember3T( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3 );
};


template< class Receiver, class CallObjType
,class TP1Fn,class TP2Fn,class TP3Fn
,class TP1,class TP2,class TP3,class ReturnType >
inline ECallbackCallerMember3T< Receiver, 
                                       ReturnType (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn)
                                       ,TP1,TP2,TP3 >
eCbCreator( ECallback3T<TP1,TP2,TP3> *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn) )
{
  typedef ReturnType (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn);

  return ECallbackCallerMember3T< Receiver, MemberCallback
                                ,TP1,TP2,TP3>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3>
inline 
ECallback3T<TP1,TP2,TP3>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3>
inline 
void
ECallback3T<TP1,TP2,TP3>::operator ()(TP1 p1,TP2 p2,TP3 p3) const
{
   _fnCaller( *this, p1,p2,p3 );
}


template<class TP1,class TP2,class TP3>
inline
ECallback3T<TP1,TP2,TP3>::ECallback3T( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3 >
inline
ECallbackCallerFn3T< Function,TP1,TP2,TP3 >::ECallbackCallerFn3T( Function fnCallback ) :
    ECallback3T<TP1,TP2,TP3>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3 >
inline void
ECallbackCallerFn3T< Function,TP1,TP2,TP3 >::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3 )
{
   ((Function)cb._fnCallback)(p1,p2,p3);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3 >
inline 
ECallbackCallerMember3T<Receiver,Member,TP1,TP2,TP3>::ECallbackCallerMember3T( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback3T<TP1,TP2,TP3>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3 >
inline void
ECallbackCallerMember3T<Receiver,Member,TP1,TP2,TP3>::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)(p1,p2,p3);
}

#endif // !defined(AFX_ECALLBACK3T_H__B78AA25C_D476_11D2_BD55_0040053AD648__INCLUDED_)
