// EDirectory.h: interface for the EDirectory class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EDIRECTORY_H__3CF78423_CCA3_11D2_9206_004005446EE6__INCLUDED_)
#define AFX_EDIRECTORY_H__3CF78423_CCA3_11D2_9206_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "EFilename.h"
#include "EString.hpp"

typedef EObjDynArrayT<EFilename> EFilenameList;


class EDirectory  
{
public:
  /*! Constructs a directory.
   */
	EDirectory();

  /*! Constructs a directory with a given name.
   * @param strName Name of the directory. 
   */
	EDirectory( const EString &strName );

  /*! Destructor.
   */
	virtual ~EDirectory();

  /*! Set the name of the directory.
   * @param strName Name of the directory. 
   */
  void setName( const EString &strName );

  /*! Get the name of the directory.
   * @return Name of the directory. The directory name always ends with a
   * '\'. Example: "C:\Test\MyPath" becomes "C:\Test\MyPath\" and
   * "C:\" stays the same.
   */
  const EString &name() const;

  /*! Set the directory name to its parent.
   */
  void setToParent();

  /*! Find a list of file in the directory.
   * @param listFilename Filename list.
   * @param strFilter Filter apply for the search in this directory.
   *        To find all files and directories use the filter "*.*"
   */
  void findFile( EFilenameList &listFilename,
                 const EString &strFilter = _T( "*.*" ) ) const;

  /*! Tests if the directory is the root directory.
   * @return \c true if the directory is the root directory, \c false otherwise.
   */
  bool isRoot() const;

private:
  EString _strName;
};


inline bool 
EDirectory::isRoot() const
{
  return EFilename( _strName ).isRootDirectory();
}

#endif // !defined(AFX_EDIRECTORY_H__3CF78423_CCA3_11D2_9206_004005446EE6__INCLUDED_)
