// EFilename.cpp: implementation of the EFilename class.
//
//////////////////////////////////////////////////////////////////////

#include "usuel.hpp"
#include "EFilename.h"
#include "emath.hpp"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EFilename::EFilename()
{

}

EFilename::~EFilename()
{

}


EFilename::EFilename( const EString &strFilename )
{
  setFilename( strFilename );
}


void 
EFilename::setFilename( const EString &strFilename )
{
  _strFilename = strFilename;
}


bool 
EFilename::isDirectory() const
{
  if ( _strFilename.length() )
  {
    TCHAR cLast = _strFilename.lastChar();
    return cLast == '/'  ||  cLast == '\\';
  }
  return false;
}


EString 
EFilename::name() const
{
  SL nPosLast;
  if ( isDirectory() )
    nPosLast = -2;
  else
    nPosLast = -1;

  SL nPosSlash = _strFilename.posLast( '\\', nPosLast );
  nPosSlash = emax( nPosSlash,
                    _strFilename.posLast( '/', nPosLast ) );
  if ( nPosSlash >= 0 )
  {
    SL nLength = _strFilename.length();
    nLength += nPosLast +1;
    nLength -= nPosSlash +1;
    return _strFilename.mid( nPosSlash +1, 
                             nLength );
  }
  
  return EString();  
}


bool 
EFilename::isRootDirectory() const
{
  if ( !isDirectory() )
    return false;

  // First '/' is also the last ? Then it is the root directory.
  SL nPosBackslash = _strFilename.pos( 0, '\\' );
  SL nPosSlash = _strFilename.pos( 0, '/' );
  if ( ( nPosBackslash >= 0  &&  nPosBackslash == _strFilename.posLast( '\\' ) )  ||
       ( nPosSlash >= 0  &&  nPosSlash == _strFilename.posLast( '/' ) ) )
  {
    return true;
  }

  return false;
}


EString 
EFilename::ext() const
{
  SL nPosDot = _strFilename.posLast( '.' );
  SL nPosDir = emax( _strFilename.posLast( '\\' ),
                     _strFilename.posLast( '/' ) );
  if ( nPosDot > nPosDir  &&  nPosDot >= 0 )
    return _strFilename.mid( nPosDot+1, -1 );
  return EString();
}


void 
EFilename::setExt( const EString strExt )
{
  EASSERT( !isDirectory() );

  SL nPosDot = _strFilename.posLast( '.' );
  if ( nPosDot > 0 )
    _strFilename.truncate( nPosDot );

  _strFilename += '.';
  _strFilename += strExt;
}
