// EFilename.h: interface for the EFilename class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EFILENAME_H__3CF78425_CCA3_11D2_9206_004005446EE6__INCLUDED_)
#define AFX_EFILENAME_H__3CF78425_CCA3_11D2_9206_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "EString.hpp"


class EFilename  
{
public:
  /*! Constructs a filename.
   *
   */
	EFilename();

  /*! Constructs a filename with a name.
   * @param strFilename Name of the file.
   */
	EFilename( const EString &strFilename );

  /*! Destructor.
   */
	virtual ~EFilename();

  /*! Set the filename.
   * @param strFilename Name of the file.
   */
  void setFilename( const EString &strFilename );

  /*! Tests if the filename is a directory.
   * @return \c true if the filename is a directory. \c false if it is a file.
   *         A filename is a directory if its ends with a '\' or a '/'.
   */
  bool isDirectory() const;

  /*! Get the last component of the full filename.
   * @return Last component of the full filename. For example, "c:\prg\toto\do.txt"
   * will return "do.txt" and "c:\prg\toto\" will return "toto".
   */
  EString name() const;

  /*! Get the full filename;
   * @return The full filename is returned. For example, "c:\prg\toto\do.txt" is
   * returned without change.
   */
  const EString &filename() const;

  /*! Get the extension of the filename.
   * @return Extension of the filename. For example, "c:\prg\toto.txt" returned "txt".
   */
  EString ext() const;

  /*! Set the extension of the filename.
   * @param strExt New extension of the filename. For example, "c:\toto.txt" with 
   *               \a strExt="html" will become "c:\toto.html".
   */
  void setExt( const EString strExt );

  /*! Tests if the directory is the root directory.
   * @return \c true if the directory is the root directory, \c false otherwise.
   */
  bool isRootDirectory() const;

private:
  EString _strFilename;
};


inline const EString &
EFilename::filename() const
{
  return _strFilename;
}


#endif // !defined(AFX_EFILENAME_H__3CF78425_CCA3_11D2_9206_004005446EE6__INCLUDED_)
