// EHashValBool.h: interface for the EHashValBool class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EHASHVALBOOL_H__FDCB40E2_DFC8_11D2_BD6F_0040053AD648__INCLUDED_)
#define AFX_EHASHVALBOOL_H__FDCB40E2_DFC8_11D2_BD6F_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// Includes:
// #####################Last change: 24/07/98 21h49mn#########################
#include "EDynArray.h"

// Constantes:
// #####################Last change: 24/07/98 21h49mn#########################


// Macros:
// #####################Last change: 24/07/98 21h49mn#########################


// Types:
// #####################Last change: 24/07/98 21h49mn#########################

template<class Key, class KeyArg>
class EHashValBoolIteratorT;


/* Hash Table.
 * Key Type of the key, must have a numeric (of type UL) value and the operator ==.
 * Item Type of the item associated with a key, for storage & return value.
 * KeyArg and ItemArg are the as Key & Item excepts that they are used when
 * passing paramaters (usually it is a reference on Key and Item type).
 *
 * To use this class you must derivate it and overide the virtual method
 * getKey() that compute a integer value based on the key and compareKey()
 * that compare two keys.
 *
 * Exemple of use:
 * Here we want to use an hash table to compute frequency of words in a text.
 * Therefore the key is the word and the item associated with the key is an
 * integer.
 *
 * Here is the class declaration:
 *
 *   Notes: Here the type for storing word and frequency are basic type,
 * therefore they no need to use reference when passing type as parameter.
 *
 *
 *   typedef EHashVal<UL,char *,UL,char *,UL> HashWordByFrq 
 *
 *   You can now add words:
 *
 *   HashWordByFrq hash;
 *
 *   if ( !hash.lookUp( 1 ) )  // the word is not in the table !
 *     hash.add( 1, "Berto" );         // Add the word to table & frq = 1
 *
 * If you override freeAsso(), you must also override the destructor, and call
 * removeAll() on destruction, otherwise the overrided freeAsso() will not be
 * called.
 */


template<class Key, class KeyArg>
class EHashValBoolT
{
#ifdef _SUPPORT_TEMPLATE_FRIEND
  friend class EHashValBoolIteratorT<Key,KeyArg>;
#endif

public:
  /// Alias for iterator.
  typedef EHashValBoolIteratorT<Key,KeyArg> EIterator;

  /** Constructor.
   * @param nSize initial size of the table.
   */
  EHashValBoolT( SL nSize = 3 );

  // Destructor
  virtual ~EHashValBoolT();

  /** Lookup a item using its key.
   * @param key key associated to the item.
   * @return reference on item.
   * */
  bool operator[]( KeyArg key ) const;

  /** Add an item to the association table.
   * @param key associated to the item.
   * @param item item to add.
   */
  void add( KeyArg key );

  /** Remove an item from the association table.
   * @param key key associated to the item to remove.
   */
  void remove( KeyArg key );

  /// Remove all the item from the association table.
  void removeAll();

  /** Resize the table.
   * @param nSize new table size.
   */
  void resize( SL nSize );

  /** Get the size of the table.
   * @return size of the table.
   */
  SL size() const;

  /** Get the number of association in the table.
   * @return number of association in the table.
   */
  SL count() const;

  /** Test if the table is empty.
   * @return true if there is no association in the table, false otherwise.
   */
  bool isEmpty() const;

#ifdef _SUPPORT_TEMPLATE_FRIEND
private:
#endif
  struct ELCAsso
  {
    ELCAsso *_pNext;
    Key _key;
  };

  EArray<ELCAsso *> _aAsso;

  SL _nNbAsso;
  
protected:
  /** Free memory use by a key and an item (in case they use indirect data).
   * For example if key is a "char *" then you need to do "free( key )".
   * @param key  key to free
   * @param item item item to free.
   */
  virtual void freeAsso( Key &key ) {}
};


// Template Implementation:
// #####################Last change: 24/07/98 21h49mn#########################

template<class Key, class KeyArg>
EHashValBoolT<Key,KeyArg>::EHashValBoolT( SL nSize )
{
  EASSERT( nSize > 0 );
  _nNbAsso = 0;
  resize( nSize );
}


template<class Key, class KeyArg>
EHashValBoolT<Key,KeyArg>::~EHashValBoolT()
{
  removeAll();
}


template<class Key, class KeyArg>
bool EHashValBoolT<Key,KeyArg>::operator[]( KeyArg key ) const
{
  SL nIndex = (UL)(key) % _aAsso.count();
  ELCAsso *pAsso = _aAsso[ nIndex ];
  while ( pAsso )
  {
    if ( pAsso->_key == key )
      return true;
    pAsso = pAsso->_pNext;
  }
  return false;
}


template<class Key, class KeyArg>
void EHashValBoolT<Key,KeyArg>::add( KeyArg key )
{
  EASSERT( _aAsso.count() > 0 );

  SL nIndex = (UL)(key) % _aAsso.count();

  ELCAsso *pAsso = new ELCAsso();
  pAsso->_key = key;
  pAsso->_pNext = _aAsso[ nIndex ];
  _aAsso[ nIndex ] = pAsso;
  _nNbAsso++;
}


template<class Key, class KeyArg>
void EHashValBoolT<Key,KeyArg>::remove( KeyArg key )
{
  EASSERT( _aAsso.count() > 0 );

  SL nIndex = (UL)(key) % _aAsso.count();
  ELCAsso *pAsso = _aAsso[ nIndex ];
  ELCAsso **pPrevAsso = &_aAsso[ nIndex ];
  while ( pAsso )
  {
    if ( pAsso->_key == key )  // found !
    {
      *pPrevAsso = pAsso->_pNext;
      freeAsso( pAsso->_key );
      delete pAsso;
      _nNbAsso--;
      return;
    }

    pPrevAsso = &pAsso->_pNext;
    pAsso = pAsso->_pNext;
  }
  EASSERT( pAsso );
}


template<class Key, class KeyArg>
void EHashValBoolT<Key,KeyArg>::resize( SL nSize )
{
  EDynArray<ELCAsso *> aAsso2;
  SL nCpt;
  ELCAsso *pNull = ENIL;

  aAsso2.resize( nSize );
  aAsso2.fill( &pNull );

  for ( nCpt = 0; nCpt < size(); ++nCpt )
  {
    ELCAsso *pAsso = _aAsso[ nCpt ];
    while ( pAsso )
    {
      ELCAsso *pNextAsso = pAsso->_pNext;
      SL nIndex = (UL)( pAsso->_key ) % nSize;    // compute new index

      pAsso->_pNext = aAsso2[ nIndex ];              // add to head of lc
      aAsso2[ nIndex ] = pAsso;

      pAsso = pNextAsso;
    }
  }

  _aAsso = aAsso2;
}


template<class Key, class KeyArg>
inline SL EHashValBoolT<Key,KeyArg>::size() const
{
  return _aAsso.count();
}


template<class Key, class KeyArg>
inline SL EHashValBoolT<Key,KeyArg>::count() const
{
  return _nNbAsso;
}


template<class Key, class KeyArg>
void EHashValBoolT<Key,KeyArg>::removeAll()
{
  SL nIndex;

  for ( nIndex = size() -1; nIndex >= 0; --nIndex )
  {
    ELCAsso *pAsso = _aAsso[ nIndex ];
    while ( pAsso )
    {
      ELCAsso *pNextAsso = pAsso->_pNext;
      freeAsso( pAsso->_key );
      delete pAsso;
      pAsso = pNextAsso;
    }
    _aAsso[ nIndex ] = ENIL;
  }
  _nNbAsso = 0;
}


template<class Key, class KeyArg>
bool 
EHashValBoolT<Key,KeyArg>::isEmpty() const
{
  return count() == 0;
}


// Includes:
// #####################Last change: 28/03/98 20h01mn#########################
#include "EHashValBoolIt.h"

#endif // !defined(AFX_EHASHVALBOOL_H__FDCB40E2_DFC8_11D2_BD6F_0040053AD648__INCLUDED_)
