// ###########################################################################
// File: EHashValPtr.hpp
// (c)Copyright 10/1998, Baptiste Lepilleur (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef EHASHVAPTRL_HPP_INCLUDED


/* History:
 * 19/10/1998: - Creation.
 *             - BugFix: Destructor was not overrided and there freeAsso() not called.
 */

// Includes:
// #####################Last change: 24/07/98 21h49mn#########################
#include "EHashVal.hpp"

// Constantes:
// #####################Last change: 24/07/98 21h49mn#########################


// Macros:
// #####################Last change: 24/07/98 21h49mn#########################


// Types:
// #####################Last change: 24/07/98 21h49mn#########################

/* \brief Hash Table.
 * 
 * The usage of this class is similar to EHashVal except than the \a Item type
 * must be a pointer.
 *
 * Then method freeAsso is override and apply a "delete" on the item pointer.
 */
template<class Key, class Item, class KeyArg>
class EHashValPtr : public EHashVal<Key,Item,KeyArg,Item>
{
public:

  /** Constructor.
   * @param nSize initial size of the table.
   */
  EHashValPtr( SL nSize = 3 ) {};

  // Destructor
  virtual ~EHashValPtr();

protected:
  /** Free memory use by a key and an item (in case they use indirect data).
   * For example if key is a "char *" then you need to do "free( key )".
   * @param key  key to free
   * @param item item item to free.
   */
  virtual void freeAsso( Key &key, 
                         Item &item );
};


// Template Implementation:
// #####################Last change: 24/07/98 21h49mn#########################

template<class Key, class Item, class KeyArg>
void
EHashValPtr<Key,Item,KeyArg>::freeAsso( Key &key, 
                                        Item &item )
{
  if ( item )
    delete item;
}


template<class Key, class Item, class KeyArg>
EHashValPtr<Key,Item,KeyArg>::~EHashValPtr() 
{ 
  removeAll();       // Otherwise freeAsso() will not be called.
}


#define EHASHVAPTRL_HPP_INCLUDED
#endif
