// ERef.h: interface for the ERef class.
//
//////////////////////////////////////////////////////////////////////

/* HST:
 * 18/10/1998: -BugFix: Forgot override operator =(const EBaseRef &).
 * 19/10/1998: - Add cast operator : (Type*).
 * 19/10/1998: - Add == operator.
 */

#if !defined(AFX_EREF_H__A1E5DBDF_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
#define AFX_EREF_H__A1E5DBDF_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EBaseRef.h"

// Build the name of the reference class.
#define E_REFNAME( ClassName ) ClassName##Ref

// Declare a new reference class.
#define EDECL_REF( Type ) \
  typedef ERef<##Type##> E_REFNAME( Type );

/*
class  FAR_REFNAME( Type ) : public ERef<##Type##> \
{ \
public: \
  FAR_REFNAME( Type )() {} \
  FAR_REFNAME( Type )( Type *p ) : ERef<##Type##>( p ) {} \
  \
  FAR_REFNAME( Type )( const FAR_REFNAME( Type )& ref ) : \
      ERef<##Type##>( ref ) {} \
}; 
*/


/*! \brief This class provides a mean to acces objects using reference counting.
 *
 * @param Type Type of object referenced. \a Type must be derived from Type.
 */
template<class Type>
class ERef : public EBaseRef
{
public:
	/*! Constructs a \c NULL reference.
   */
  ERef() {}
  
  /*! Constructs a reference from a pointer to an object.
   * @param p Object that is referenced.
   */
	ERef( Type *p );

  /*! Constructs a reference from another reference.
   * @param ref Reference that is copied.
   */
  ERef( const ERef<Type>& ref );

  /*! Release the reference to the object.
   */
  ~ERef() {}

	Type& operator*();

	Type** operator&();

	Type* operator->();

	Type* operator=( Type *p );

	Type* operator=( const ERef<Type>& ref );

  operator Type *();

  /*! Tests if two references are equal.
   * @return \c true if the two reference are equal, \c false otherwise.
   */
  bool operator ==( const ERef<Type> &ref ) const;
};




template<class Type>
inline 
ERef<Type>::ERef( Type *p ) :
    EBaseRef( p )
{
}


template<class Type>
inline 
ERef<Type>::ERef( const ERef<Type>& ref ) :
    EBaseRef( ref )
{
}


template<class Type>
inline Type& 
ERef<Type>::operator*()
{
  return (Type&)EBaseRef::operator*();
}


template<class Type>
inline Type** 
ERef<Type>::operator &() 
{ 
  return (Type**)EBaseRef::operator&();
}


template<class Type>
inline Type* 
ERef<Type>::operator ->() 
{ 
  return (Type *)EBaseRef::operator->();
}


template<class Type>
inline  Type *
ERef<Type>::operator =( Type *p )
{
  if ( _p )
    _p->decRef();
  _p = p;
  if ( p )
    p->incRef();
  return p;
}


template<class Type>
inline  Type *
ERef<Type>::operator=( const ERef<Type>& ref )
{
  return (Type *)EBaseRef::operator=( ref );
}


template<class Type>
inline
ERef<Type>::operator Type *()
{
  return (Type *)_p;
}


template<class Type>
inline bool 
ERef<Type>::operator ==( const ERef<Type>& ref ) const
{
  return _p == ref._p;
}


#endif // !defined(AFX_EREF_H__A1E5DBDF_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
