// EStoreObjectDef.h: interface for the EStoreObjectDef class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESTOREOTDEF_H__27A1D184_7332_11D3_BE8F_0040053AD648__INCLUDED_)
#define AFX_ESTOREOTDEF_H__27A1D184_7332_11D3_BE8F_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EHashBiT.h"
#include "EStoreOtPrp.h"

typedef UL EStoreObjectType;

const EStoreObjectType esotpNone =0;
const EStoreObjectType esotpFirst = ( esotpNone +1 );



class EStoreOtDef  
{
public:
  struct PrpTypeInfo
  {
    PrpTypeInfo();
    PrpTypeInfo( const EString &strPrpType,
                 EStoreOtPrp::PrpValueType pvtpPrp );

    EString _strPrpTypeName;
    EStoreOtPrp::PrpValueType _pvtpPrp;
  };


	EStoreOtDef();
	virtual ~EStoreOtDef();

  /** Find an object type using its name.  
   * @param strObjectType [in] Name of the object type.
   * @return Matching object type. \c esotpNone if none is found. The match is
   *         case insensitive.
   */
  EStoreObjectType objectType( const EString &strObjectType ) const;

  /** Find an object type name using its type.
   * @param esotpObject Object type.
   * @return Reference on the matching object type name. \c nullString if none.
   */
  const EString &objectTypeName( EStoreObjectType esotpObject ) const;

  /** Find a property type using its name.  
   * @param strPrpType [in] Name of the property type.
   * @return Matching property type. \c esptpNone if none is found. The match is
   *         case insensitive.
   */
  EStorePrpType prpType( const EString &strPrpType ) const;

  /** Find a property type info using its name.  
   * @param strPrpType [in] Name of the property type.
   * @return Matching property type. \c esptpNone if none is found. The match is
   *         case insensitive.
   */
  const PrpTypeInfo &prpTypeInfo( const EString &strPrpType ) const;

  /** Find a property type name using its type.
   * @param esptpPrp Property type.
   * @return Reference on the matching property type name. \c nullString if none.
   */
  const EString &prpTypeName( EStorePrpType esptpPrp ) const;

  /** Add an object type.
   * @param esotpObject Object type.
   * @param strObjectType Name of the object type.
   */
  void addObjectType( EStoreObjectType esotpObject,
                      const EString &strObjectType );

  /** Add a property type.
   * @param esptpPrp Property type.
   * @param strPrpType Name of the property type.
   * @param pvtpPrp Property value type. If \c pvtpNone the value of the property
   *                may be of any type, otherwise the value of the property must
   *                be of the specified type.
   */
  void addPrpType( EStorePrpType esptpPrp,
                   const EString &strPrpType,
                   EStoreOtPrp::PrpValueType pvtpPrp = EStoreOtPrp::pvtpNone );

private:

  class PrpInfoKeyProvider : public EHashKeyProviderT<const PrpTypeInfo &>
  {
    public:
      virtual UL getKey( const PrpTypeInfo &key ) const
      {
        return key._strPrpTypeName.imakeHashKey();
      }

      virtual bool compareKey( const PrpTypeInfo &key1, 
                               const PrpTypeInfo &key2 ) const
      {
        return key1._strPrpTypeName == key2._strPrpTypeName;
      }
  };

  typedef EHashBiT<EStoreObjectType,EString,EStoreObjectType,const EString &,
                   EStringNcsKeyProviderT<const EString> > HashBiDefObject;

  typedef EHashBiT<EStorePrpType,PrpTypeInfo,EStorePrpType,const PrpTypeInfo &,
                   PrpInfoKeyProvider > HashBiDefPrp;

  HashBiDefObject _bhashDefObject;
  HashBiDefPrp _bhashDefPrp;
};  




inline
EStoreOtDef::PrpTypeInfo::PrpTypeInfo() :
    _pvtpPrp( EStoreOtPrp::pvtpNone )
{
}


inline
EStoreOtDef::PrpTypeInfo::PrpTypeInfo( const EString &strPrpType,
                     EStoreOtPrp::PrpValueType pvtpPrp ) :
    _strPrpTypeName( strPrpType ),
    _pvtpPrp( pvtpPrp )
{
}






#endif // !defined(AFX_ESTOREOTDEF_H__27A1D184_7332_11D3_BE8F_0040053AD648__INCLUDED_)
