// EStringHelperT.h: interface for the EStringHelperT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESTRINGHELPERT_H__992AFCE6_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
#define AFX_ESTRINGHELPERT_H__992AFCE6_3D03_11D3_BE1D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <ctype.h>
#include "ECharTraitT.h"

/*! \class EStringHelper<CharType>
 * \include "EStringHelperT.h"
 * \brief Template use to manipulate string.
 */
template<class CharType>
class EStringHelperT
{
public:
  /*! This string type.
   */
  typedef const CharType *ThisCsz;

  /*! This character type.
   */
  typedef CharType ThisChar;

  /*! Chararacter trait used for this type.
   */
  typedef ECharTraitT<CharType> ThisCharTrait;

  /*! Make a case-sensitive hash-key for a string.
   * @param cszString String from which the hash-key is computed.
   * @return Case-sensitive hash-key.
   */
  static UL makeHashKey( ThisCsz cszString );

  /*! Make a case-insensitive hash-key for a string.
   * @param cszString String from which the hash-key is computed.
   * @return Case-insensitive hash-key.
   */
  static UL imakeHashKey( ThisCsz cszString );

  /*! Duplicate a string.
   * @param cszString String to duplicate.
   * @return Pointer on the duplicated string (allocated with new []).
   */
  static ThisSz duplicate( ThisCsz cszString );

};

typedef EStringHelperT<char> EAStringHelper;
typedef EStringHelperT<wchar_t> EWStringHelper;

#ifdef UNICODE
  typedef EWStringHelpher EStringHelper;
#else  // UNICODE
  typedef EAStringHelper EStringHelpher;
#endif // UNICODE



// Template EStringHelperT implementation
// --------------------------------------

template<class CharType>
UL 
EStringHelper<CharType>::makeHashKey( ThisCsz cszString )
{
  UL ulKey = 0;
  CharType cChar;

  if ( cszString )
  {
    while ( cChar = *cszString++ )
    {
      ulKey =  ( ulKey << 5 ) + ( ulKey >> (32-5) ) + (UL)cChar;
    }
  }

  return ulKey;
}


template<class CharType>
UL 
EStringHelper<CharType>::imakeHashKey( ThisCsz cszString )
{
  UL ulKey = 0;
  CharType cChar;

  if ( cszString )
  {
    while ( cChar = ThisCharTrait::lower( *cszString++ ) )
    {
      ulKey =  ( ulKey << 5 ) + ( ulKey >> (32-5) ) + (UL)cChar;
    }
  }

  return ulKey;
}


template<class CharType>
ThisSz 
EStringHelper<CharType>::duplicate( ThisCsz cszString )
{
  SL nLength = ThisCharTrait::length( cszString ) +1;
  ThisSz *szDst = new CharType[ nLength ];
  return ThisCharTrait::copy( cszString, szDSt, nLength );
}


#endif // !defined(AFX_ESTRINGHELPERT_H__992AFCE6_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
