// ###########################################################################
// File: EMIODEVI.HPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// version 0.3
// ###########################################################################


#ifndef EMIODEVI_HPP_INCLUDED
#define EMIODEVI_HPP_INCLUDED

/* HST:
 * - 29/03/98: Destructor is now virtual and call ~EIODevice().
 * - 24/07/98: change _aDevices declaration due to EArray template definition change.
 */

// Includes:
// #####################Last change: 23/08/97  8h00mn#########################

#include "eiofiltr.hpp"
#include "earray.hpp"

// Constantes:
// #####################Last change: 23/08/97  8h00mn#########################


// Macros:
// #####################Last change: 23/08/97  8h00mn#########################


// Types:
// #####################Last change: 23/08/97  8h00mn#########################

class EMultiIODevice : public EIODevice
{
  public:
    /// Default Constructor.
    EMultiIODevice();

    /// Destructor.
    virtual ~EMultiIODevice();

    /** Add a device.
     * Only one instance of the device can be present in a multi I/O device.
     * If the device is already present, it will not be added.
     * @param device device to add.
     */
    void addDevice( EIODevice *device );

    /** Remove a device.
     * Remove a device.
     * @param device device to remove.
     */
    void removeDevice( EIODevice *device );

    /** Return the number of devices.
     * @return number of devices.
     */
    int count() const;

    /** Open the devices.
     * Open all the registred devices using mode \a _mode.
     * @param _mode mode use to open the device
     */
    void open( int _mode );

    /** Close the devices.
     * Close all the registred devices.
     */
    void close();

    /** Flush the devices.
     * Flush all the registred devices.
     */
    void flush();

    /** Always return 0.
     * @return 0.
     */
    SL size() const;

    /** Always return 0.
     * @return 0
     */
    SL at() const;

    /** Set the position of devices.
     * Set the position of all the registred devices (index); to \a pos.
     * @param pos new position of the index.
     * @exception EXCIODevice on error.
     */
    void at( SL pos);

    /** always return false.
     * @return false.
     */
    bool atEnd() const;

    /** Do nothing.
     * @param data dummy.
     * @param len dummy.
     * @return always 0.
     */
    SL readBlock( void *data, SL len);

    /** Write a block of byte on devices.
     * Write \a len Byte of buffer \a data on each registred devices.
     * @param data buffer where data are read.
     * @param len number of bytes of data to write.
     * @exception EXCIODevice is case of grave error.
     */
    void writeBlock( const void *data, SL len );

    /** Always return 0.
     * @return 0.
     */
    UB getByte();

    /** Write a Byte on each devices.
     * @param c byte to write.
     * @exception EIODevice on error.
     */
    void putByte( UB c );

    /** Push-back a byte on each devices.
     * @param c byte to push-back.
     * @exception EIODevice on error.
     */
    void ungetByte( UB c );


  private:

    /// array of devices.
    EArray<EIODevice *,true> _aDevices;
};

// inlines:
// #####################Last change: 23/08/97  8h00mn#########################


// Prototypes:
// #####################Last change: 23/08/97  8h00mn#########################



#endif


// ########################### EOF EMIODEVI.HPP ##############################
// ########################### EOF EMIODEVI.HPP ##############################
// ########################### EOF EMIODEVI.HPP ##############################

