// TfwkArrayItT.h: interface for the TfwkArrayItT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKARRAYITT_H__1D7D9219_4132_11D3_BE28_0040053AD648__INCLUDED_)
#define AFX_TFWKARRAYITT_H__1D7D9219_4132_11D3_BE28_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkCommon.h"

template<class Obj>
class TfwkArrayT;

/*! \class TfwkArrayItT<Array,Obj>
 * \nclude "TfwkArrayItT.h"
 * \brief This templates provides a iterator on a array.
 * @param Array Type of array the iterator iterates on. Should be of type TfwkArrayItT<Obj>.
 * @param Obj Type of the object the iterator iterate on.
 */
template<class Array, class Obj>
class TfwkArrayItT
{
public:

  /*!
   * Constructs an iterator on a array.
   * @param array Array the iterator iterate on. Must be of type TfwkArrayItT<Obj>.
   */
	TfwkArrayItT( Array &array );

  /*!
   * Destructor.
   */
  virtual ~TfwkArrayItT() {}


  /*!
   * Sets the iterator on the first item of the array.
   */
  virtual void begin();

  /*!
   * Sets the iterator after the last item of the array.
   */
  virtual void end();

  /*!
   * Steps the iterator to the next item.
   */
  virtual void next();

  /*!
   * Steps the iterator to the previous item.
   */
  virtual void prev();

  /*!
   * Gets the current position of the iterator.
   * @return Current position of the iterator (zero based).
   */
  virtual int pos() const;

  /*!
   * Gets a reference on the item that is pointed by the iterator.
   * @return Reference on the item at the current position of the iterator.
   */
  virtual Obj &item() const;

  /*! 
   * Tests if the iterator is past the last item of the array.
   * @return \c true if the iterator is past the last item of the array, otherwise
   *         returns \c false.
   */
  virtual bool isEnd() const;

  /*!
   * Gets the number of items in the array.
   * @return Number of items in the array.
   */
  int length() const;

private:
  int _nIndex;
  Array *_pArray;
};


// Template implementation: TfwkArrayItT
// -------------------------------------

template<class Array, class Obj>
TfwkArrayItT<Array,Obj>::TfwkArrayItT( Array &array ) :
    _pArray( &array ),
    _nIndex( 0 )
{
}


template<class Array, class Obj>
inline void 
TfwkArrayItT<Array,Obj>::begin()
{
  _nIndex = 0;
}


template<class Array, class Obj>
inline void 
TfwkArrayItT<Array,Obj>::end()
{
  _nIndex = _pArray->length();
}


template<class Array, class Obj>
inline void 
TfwkArrayItT<Array,Obj>::next()
{
  if ( !isEnd() )
    ++_nIndex;
}

template<class Array, class Obj>
inline void 
TfwkArrayItT<Array,Obj>::prev()
{
  TFWKASSERT( _nIndex >= 0 );
  if ( _nIndex )
    --_nIndex;
}


template<class Array, class Obj>
inline int 
TfwkArrayItT<Array,Obj>::pos() const
{
  TFWKASSERT( _nIndex >= 0 );
  return _nIndex;
}


template<class Array, class Obj>
inline Obj &
TfwkArrayItT<Array,Obj>::item() const
{
  TFWKASSERT( _nIndex >= 0  &&  _nIndex < length() );
  return _pArray->at( pos() );
}


template<class Array, class Obj>
inline bool 
TfwkArrayItT<Array,Obj>::isEnd() const
{
  return pos() >= length();
}


template<class Array, class Obj>
inline int
TfwkArrayItT<Array,Obj>::length() const
{
  return _pArray->length();
}


#endif // !defined(AFX_TFWKARRAYITT_H__1D7D9219_4132_11D3_BE28_0040053AD648__INCLUDED_)
