// TfwkExc.h: interface for the TfwkExc class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKEXC_H__1D7D9217_4132_11D3_BE28_0040053AD648__INCLUDED_)
#define AFX_TFWKEXC_H__1D7D9217_4132_11D3_BE28_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

/*!
 * Macro used to throw an exception without information.
 * @param nReason Reason of the exception. See TfwkExc::TfwkReason.
 * \sa TfwkExc.
 */
#define TFWKTHROW( nReason ) \
  throw new TfwkExc( nReason, __LINE__, __FILE__ );

/*!
 * Macro used to throw an exception with information.
 * @param nReason Reason of the exception. See TfwkExc::TfwkReason.
 * @param szInfo Information about the exception. Must be static.
 * \sa TfwkExc.
 */
#define TFWKTHROWINFO( nReason, szInfo ) \
  throw new TfwkExc( nReason, __LINE__, __FILE__, szInfo );

/*!
 * \class TfwkExc
 * \include "TfwkExc.h"
 * \brief This class represents an exception.
 */
class TFWK_UI_EXPORT TfwkExc  
{
public:
  /*!
   * Exception reason.
   */
  enum TfwkReason
  {
    /// No reason.
    trNone =0,
    /// Out of memory
    trOutOfMemory,
    /// Assertion failed. The information contains the asserted expression.
    trAssertionFailed,
  };

  /*!
   * Constructs an exception.
   *
   * @param trReason Reason of the exception.
   * @param nLine Line from which the exception was thrown.
   * @param szSource Name of the source file from which the exception was thrown.
   * @param szInfo Information about the exception (not copied, must be static).
   *               Can be \c NULL.
   */
	TfwkExc( TfwkReason trReason, 
           int nLine, 
           const char *szSource,
           const char *szInfo =NULL );

  /*! 
   * Destructor.
   */
  virtual ~TfwkExc() {}

  /*!
   * Gets the reason of the exception.
   * @return Reason of the exception.
   */
  TfwkReason reason() const;

  /*!
   * Gets the line from which the exception was thrown.
   * @return Line from which the exception was thrown.
   */
  int line() const;

  /*! 
   * Gets the name of the source file from which the exception was thrown.
   * @return Name of the source file from which the exception was thrown.
   */
  const char *source() const;

  /*!
   * Gets information about the exception.
   * @return Information about the exception.
   */
  const char *info() const;

private:
  /*!
   * Reason of the exception.
   */
  TfwkReason _trReason;

  /*! 
   * Line from which the exception was thrown.
   */
  int _nLine;

  /*!
   * Source file from which the exception was thrown.
   */
  const char *_szSource;

  /*!
   * Information about the exception.
   */
  const char *_szInfo;
};


inline
TfwkExc::TfwkExc( TfwkReason trReason, 
                  int nLine, 
                  const char *szSource,
                  const char *szInfo ) :
    _trReason( trReason ),
    _nLine( nLine ),
    _szSource( szSource ),
    _szInfo( szInfo )
{
}


inline TfwkExc::TfwkReason 
TfwkExc::reason() const
{
  return _trReason;
}


inline int 
TfwkExc::line() const
{
  return _nLine;
}


inline const char *
TfwkExc::source() const
{
  return _szSource ? _szSource : "";
}


inline const char *
TfwkExc::info() const
{
  return _szInfo ? _szInfo : "";
}


#endif // !defined(AFX_TFWKEXC_H__1D7D9217_4132_11D3_BE28_0040053AD648__INCLUDED_)
