// TfwkInterface.h: interface for the TfwkInterface class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKINTERFACE_H__D4F96021_4386_11D3_BE31_0040053AD648__INCLUDED_)
#define AFX_TFWKINTERFACE_H__D4F96021_4386_11D3_BE31_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkCommon.h"
#include "TfwkUnitTest.h"

class TfwkUnitTestInfo;


/*! \class TfwkInterface
 * \include "TfwkInterface.h"
 * \brief This class is the base class for all client communication interface classes.
 */
class TfwkInterface  
{
public:
  /*!
   * Constructs an interface.
   * @param szName Name of the source. Must not be \c NULL.
   */
	TfwkInterface( const char *szName = "Default Source" );

  /*!
   * Destructor.
   */
  virtual ~TfwkInterface() {}

  /*!
   * Sets the name of the source.
   * @param szName Name of the source. Must not be \cNULL or an empty string.
   */
  void setName( const char *szName );

  /*!
   * Gets the name of the source.
   */
  const char *name() const;

  /*!
   * Sends the list of all test class & test method to the server.
   */
  virtual void sendAllTest();

private:
  /*!
   * Name of the source.
   */
  const char *_szName;
};




// Inline : TfwkInterface
// ----------------------

inline 
TfwkInterface::TfwkInterface( const char *szName ) :
    _szName( szName )
{
  TFWKASSERT( szName );   // The name of the source must not be NULL.
}


inline const char *
TfwkInterface::name() const
{
  return _szName;
}


inline void 
TfwkInterface::setName( const char *szName )
{
  TFWKASSERT( szName  &&  szName[0] );
  _szName = szName;
}


#endif // !defined(AFX_TFWKINTERFACE_H__D4F96021_4386_11D3_BE31_0040053AD648__INCLUDED_)
