// TfwkUiCfgReader.cpp: implementation of the TfwkUiCfgReader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Resource.h"
#include "TfwkUi.h"
#include "TfwkUiCfgReader.h"
#include "TfwkUiCfgBuilder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TfwkUiCfgReader::TfwkUiCfgReader( CFile &file, 
                                  TfwkUiCfgBuilder &builder ) :
    _nLine( 0 ),
    _idsError( IDS_TFWKUI_CFG_SYNTAX_ERROR ),
    _pBuilder( &builder )
{
  _pArchive = new CArchive( &file, CArchive::load );
}


TfwkUiCfgReader::~TfwkUiCfgReader()
{
  if ( _pArchive )
    delete _pArchive;
}


TfwkUiCfgReader::TokenType 
TfwkUiCfgReader::ReadToken()
{
  CArchive &ar = archive();
  TokenType tktp = tktpNone;

  _idsError = IDS_TFWKUI_CFG_SYNTAX_ERROR;

  do
  {
    ++_nLine;

    // Reads the line from the stream
    CString strLine;
    bool bEof = ar.ReadString( strLine ) ? true : false;

    if ( !bEof )
      tktp = tktpEof;
    else
    {
      strLine.TrimLeft();
      strLine.TrimRight();

      TCHAR cHead =0;
      if ( !strLine.IsEmpty() )
        cHead = strLine[ 0 ];

      bool bSkipLine = strLine.IsEmpty()  ||
                       cHead == _T( '#' );
      
      if ( !bSkipLine )   // This is a valid line
      {
        if ( cHead == _T( '[' ) ) // This is a tag
        {
          int nPosEnd = strLine.Find( _T( ']' ) );
          if ( nPosEnd == -1  ||                    // no ']' or not at the end
               nPosEnd != strLine.GetLength() -1 )
            tktp = tktpError;
          else                                      // '[' and ']' found
          {
            _strTag = strLine.Mid( 1, strLine.GetLength() -2 );
            _strTag.MakeLower();

            if ( _strTag == _T( "suite" ) )
              tktp = tktpSuite;
            else if ( _strTag == _T( "endsuite" ) )
              tktp = tktpEndSuite;
            else if ( _strTag == _T( "test" ) )
              tktp = tktpTest;
            else
              tktp = tktpError;
          }
        }
        else    // line does not start with '['
        {       // It must be a key:   NAME = VALUE
          if ( !_key.Parse( strLine ) )
            tktp = tktpError;
          else
            tktp = tktpKey;
        }
      }
    }
  }
  while ( tktp == tktpNone );

  return tktp;
}



TfwkUiCfgReader::TokenType
TfwkUiCfgReader::ParseSuiteTree()
{
  TokenType tktp = ReadToken();
  do
  {
    if ( tktp == tktpNone )
      tktp = ReadToken();

    switch ( tktp )
    {
    case tktpSuite:
      tktp = ParseSuite();
      break;
     
    case tktpEndSuite:
      {
        if ( !builder().BuildEndSuite() )
        {
          _idsError = IDS_TFWKUI_CFG_NO_MATCHING_SUITE_TAG;
          tktp = tktpError;
        }
        tktp = tktpNone;
      }
      break;
    case tktpTest:
      tktp = ParseTest();
      break;
    case tktpEof:
    case tktpError:
      break;
    default:
      ASSERT( FALSE );
      tktp = tktpError;
      break;
    }
  }
  while ( tktp != tktpEof  &&  
          tktp != tktpError );

  return tktp;
}


TfwkUiCfgReader::TokenType
TfwkUiCfgReader::ParseSuite()
{
  TokenType tktp = ReadToken();
  if ( tktp == tktpKey  &&  _key.IsName( _T( "NAME" ) ) )
  {
    builder().BuildSuite( _key.Value() );
    return tktpNone;
  }

  _idsError = IDS_TFWKUI_CFG_EXPECTED_NAME;
  return tktpError;
}


TfwkUiCfgReader::TokenType
TfwkUiCfgReader::ParseTest()
{
  TokenType tktp = ReadToken();
  if ( tktp != tktpKey  ||  !_key.IsName( _T( "CLASS" ) ) )
  {
    _idsError = IDS_TFWKUI_CFG_EXPECTED_CLASS;
    return tktpError;
  }

  CString strClass = _key.Value();

  tktp = ReadToken();
  if ( tktp != tktpKey  ||  !_key.IsName( _T( "METHOD" ) ) )
  {
    _idsError = IDS_TFWKUI_CFG_EXPECTED_METHOD;
    return tktpError;
  }

  builder().BuildTest( strClass, _key.Value() );
  return tktpNone;
}
