// TfwkUiCfgReader.h: interface for the TfwkUiCfgReader class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUICFGREADER_H__EDB173B1_41EF_11D3_BE2B_0040053AD648__INCLUDED_)
#define AFX_TFWKUICFGREADER_H__EDB173B1_41EF_11D3_BE2B_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkUiCfgKey.h"
class TfwkUiCfgBuilder;

class TfwkUiCfgReader  
{
public:
  enum TokenType
  {
    tktpNone =0,
    tktpEof,
    tktpError,
    tktpSuite,
    tktpEndSuite,
    tktpTest,
    tktpKey,
  };


  /*!
   * Constructs a configuration reader.
   * @param file [I/O] File the configuration is read from.
   */ 
	TfwkUiCfgReader( CFile &file, 
                   TfwkUiCfgBuilder &builder );

  /*!
   * Destructor.
   */
	virtual ~TfwkUiCfgReader();

  /*! Gets the builder.
   * @return Reference on the builder.
   */
  TfwkUiCfgBuilder &builder();

  /*!
   * Reads a tree of suite.
   */
  TokenType ParseSuiteTree();

  /*! 
   * Gets last error resource string id.
   * @return Last error resource string id.
   */
  UINT LastError() const;

  /*!
   * Gets current line.
   */
  int Line() const;

protected:
  

  /*!
   * Reads a line of text from the file.
   * @param [Out] Line that has been read.
   * @return \c false if the end of the file has been reached, otherwise
   *         returns \c true.
   */
  TokenType ReadToken();

  /*!
   * Reads a suite.
   */
  TokenType ParseSuite();

  /*!
   * Reads a test
   */
  TokenType ParseTest();

private:
  /*!
   * Gets a reference on the file text is read from.
   */
  CArchive &archive();

  /*!
   * File text is read from.
   */
  CArchive *_pArchive;

  /*!
   * Current tag name.
   */
  CString _strTag;

  /*!
   * Current configuration key.
   */
  TfwkUiCfgKey _key;

  /*!
   * Current line number.
   */
  int _nLine;

  /*!
   * Associated builder.
   */
  TfwkUiCfgBuilder *_pBuilder;

  /*!
   * Error
   */
  UINT _idsError;
};


inline CArchive &
TfwkUiCfgReader::archive()
{
  ASSERT( _pArchive );
  return *_pArchive;
}


inline TfwkUiCfgBuilder &
TfwkUiCfgReader::builder()
{
  ASSERT( _pBuilder );
  return *_pBuilder;
}


inline UINT 
TfwkUiCfgReader::LastError() const
{
  return _idsError;
}


inline int 
TfwkUiCfgReader::Line() const
{
  return _nLine;
}

#endif // !defined(AFX_TFWKUICFGREADER_H__EDB173B1_41EF_11D3_BE2B_0040053AD648__INCLUDED_)
