// TfwkUiCfgWriter.cpp: implementation of the TfwkUiCfgWriter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TfwkUi.h"
#include "TfwkUiCfgWriter.h"
#include "TfwkUiCfg.h"
#include "TfwkUiCfgKey.h"
#include "TfwkUiSuite.h"
#include "TfwkUiTest.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TfwkUiCfgWriter::TfwkUiCfgWriter( CFile &file,
                                 TfwkUiCfg &config ) :
    _pConfig( &config )
{
    _pArchive = new CArchive( &file, CArchive::store );
}

TfwkUiCfgWriter::~TfwkUiCfgWriter()
{
  if ( _pArchive )
    delete _pArchive;
}


void 
TfwkUiCfgWriter::Save()
{
  TfwkUiSuite &suite = _pConfig->RootSuite();
  _nSuiteDepth =0;
  SaveSuiteChildren( suite );
}


void 
TfwkUiCfgWriter::SaveSuite( TfwkUiSuite &suite )
{
  WriteTag( _T( "Suite" ) );
  WriteKey( _T( "NAME" ), suite.Name() );
  SaveSuiteChildren( suite );
  WriteTag( _T( "EndSuite" ) );
  WriteLine();
}


void 
TfwkUiCfgWriter::SaveSuiteChildren( TfwkUiSuite &suite )
{
  ++_nSuiteDepth;

  TfwkUiSuite::SuiteIt itSuite = suite.ItSuite();
  for ( itSuite.begin(); !itSuite.isEnd(); itSuite.next() )
  {
    SaveSuite( *itSuite.item() );
  }

  TfwkUiSuite::TestIt itTest = suite.ItTest();
  for ( itTest.begin(); !itTest.isEnd(); itTest.next() )
  {
    SaveTest( *itTest.item() );
  }

  --_nSuiteDepth;
}


void 
TfwkUiCfgWriter::SaveTest( TfwkUiTest &test )
{
  WriteLine( _T( "[Test]" ) );
  WriteKey( _T( "CLASS" ), test.Class() );
  WriteKey( _T( "METHOD" ), test.Method() );
}


void 
TfwkUiCfgWriter::WriteTag( const CString &strTagName )
{
  CString str = _T( "[" );
  str += strTagName;
  str += _T( "]" );
  WriteLine( str );
}


void 
TfwkUiCfgWriter::WriteKey( const CString &strKeyName,
                           const CString &strKeyValue )
{
  TfwkUiCfgKey key( strKeyName, strKeyValue );
  CString str;
  key.Make( str );
  WriteLine( str );
}
