// TfwkUiSuite.h: interface for the TfwkUiSuite class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUISUITE_H__AC16C641_420D_11D3_BE2C_0040053AD648__INCLUDED_)
#define AFX_TFWKUISUITE_H__AC16C641_420D_11D3_BE2C_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkArrayPtrT.h"
#include "TfwkUiTest.h"
#include "TfwkUiTreeSuiteItem.h"
#include "TfwkUiTestSuiteReport.h"
class TfwkUiCfg;


class TfwkUiSuite : public TfwkUiTreeSuiteItem
{
public:
  friend class TfwkUiTest;

  typedef TfwkArrayPtrT<TfwkUiSuite *> SuiteArray;
  typedef TfwkArrayPtrT<TfwkUiTest *> TestArray;
  typedef SuiteArray::TfwkIt SuiteIt;
  typedef TestArray::TfwkIt TestIt;


  /*!
   * Constructs a suite.
   * @param pSuite [I/O] Parent suite. \c NULL for the root suite.
   * @param strName [In] Name of the suite.
   */
	TfwkUiSuite( TfwkUiSuite *pSuite,
               const CString &strName );

  /*!
   * Destructor.
   */
	virtual ~TfwkUiSuite();

  /*!
   * Gets the name of the suite.
   * @return Name of the suite.
   */
  const CString &Name() const;

  /*!
   * Sets the name of the suite.
   * @param strName [In] New suite name.
   */
  void SetName( const CString &strName );

  /*!
   * Gets the parent suite.
   * @return Pointer on the parent suite. \c NULL for the root suite.
   */
  TfwkUiSuite *ParentSuite() const;

  /*!
   * Gets an iterator on the sub-suite.
   * @return Iterator on the sub-suite.
   */
  SuiteIt ItSuite();

  /*!
   * Gets an iterator on the sub-test.
   * @return Iterator on the sub-test.
   */
  TestIt ItTest();

  /*!
   * Removes all children of the suite.
   */
  void Clear();

  /*!
   * Removes a suite.
   * @param pSuite [In] Suite to remove.
   */
  void RemoveSuite( TfwkUiSuite *pSuite );

  /*!
   * Removes a suite.
   * @param pTest [In] Test to remove.
   */
  void RemoveTest( TfwkUiTest *pTest );

  /*!
   * Tests if the item is a suite.
   * @return \c true if the item is a suite.
   */
  virtual bool IsSuite() const;

  /*!
   * Runs a test suite.
   * @param cfg Configuration.
   * @param report Report structure.
   */
  void RunTest( TfwkUiCfg &cfg,
                TfwkUiTestSuiteReport &report );

protected:
  /*!
   * Adds a suite.
   * @param pSuite [In] Suite to add.
   */
  void AddSuite( TfwkUiSuite *pSuite );

  /*!
   * Adds a test.
   * @param pTest [In] Test to add.
   */
  void AddTest( TfwkUiTest *pTest );

private:
  CString _strName;
  TfwkUiSuite *_pParentSuite;
  SuiteArray _aSuite;
  TestArray _aTest;
};


// Inline : TfwkUiSuite
// --------------------

inline TfwkUiSuite::SuiteIt 
TfwkUiSuite::ItSuite()
{
  return SuiteIt( _aSuite );
}


inline TfwkUiSuite::TestIt 
TfwkUiSuite::ItTest()
{
  return TestIt( _aTest );
}


inline const CString &
TfwkUiSuite::Name() const
{
  return _strName;
}


inline TfwkUiSuite *
TfwkUiSuite::ParentSuite() const
{
  return _pParentSuite;
}


inline bool 
TfwkUiSuite::IsSuite() const
{
  return true;
}


inline void 
TfwkUiSuite::SetName( const CString &strName )
{
  _strName = strName;
}


#endif // !defined(AFX_TFWKUISUITE_H__AC16C641_420D_11D3_BE2C_0040053AD648__INCLUDED_)
