// TfwkUiTest.h: interface for the TfwkUiTest class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUITEST_H__AC16C640_420D_11D3_BE2C_0040053AD648__INCLUDED_)
#define AFX_TFWKUITEST_H__AC16C640_420D_11D3_BE2C_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkUiSourceId.h"
#include "TfwkUiTreeSuiteItem.h"
#include "TfwkUiTestSuiteReport.h"
class TfwkUiSuite;
class TfwkUiCfg;
class TfwkUiTestCfg;

class TfwkUiTest : public TfwkUiTreeSuiteItem
{
public:
  /*!
   * Constructs a test.
   * @param pSuite [I/O] Parent suite. Must not be \c NULL.
   * @param strClass [In] Name of the class the test belong to.
   * @param strName [In] Name of the method used to run the test.
   * @param pUnitTest [I/O] Configuration Unit Test. 
   *                  Its reference count is increased. May be \c NULL.
   */
	TfwkUiTest( TfwkUiSuite *pSuite,
              const CString &strClass,
              const CString &strMethod,
              TfwkUiSourceId idSource,
              TfwkUiTestCfg *pUnitTest );

  /*!
   * Destructor.
   */
	virtual ~TfwkUiTest();

  /*!
   * Gets the class.
   * @return Class name.
   */
  const CString &Class() const;

  /*! 
   * Gets the method.
   * @return Method name.
   */
  const CString &Method() const;

  /*!
   * Tests if the item is a suite.
   * @return \c true if the item is a suite.
   */
  virtual bool IsSuite() const;

  /*!
   * Gets the parent suite.
   * @return Reference on the parent suite.
   */
  TfwkUiSuite &ParentSuite() const;

  /*!
   * Gets the full method name (class::method).
   * @param strFullName [Out] Full method name.
   */
  void FullMethodName( CString &strFullName ) const;

  /*!
   * Runs a test suite.
   * @param cfg Configuration.
   * @param report Report structure.
   */
  void RunTest( TfwkUiCfg &cfg,
                TfwkUiTestSuiteReport &report );

private:
  CString _strClass;
  CString _strMethod;
  TfwkUiSuite *_pSuite;
  TfwkUiSourceId _idSource;
  TfwkUiTestCfg *_pUnitTest;
};



inline const CString &
TfwkUiTest::Class() const
{
  return _strClass;
}


inline const CString &
TfwkUiTest::Method() const
{
  return _strMethod;
}


inline bool 
TfwkUiTest::IsSuite() const
{
  return false;
}


inline TfwkUiSuite &
TfwkUiTest::ParentSuite() const
{
  ASSERT( _pSuite );
  return *_pSuite;
}


inline void 
TfwkUiTest::FullMethodName( CString &strFullName ) const
{
  strFullName.Format( "%s::%s", 
                      (LPCTSTR)Class(), 
                      (LPCTSTR)Method() );
}


#endif // !defined(AFX_TFWKUITEST_H__AC16C640_420D_11D3_BE2C_0040053AD648__INCLUDED_)
