// TfwkUiTestCfg.h: interface for the TfwkUiTestCfg class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUITESTCFG_H__33520481_4266_11D3_BE2D_0040053AD648__INCLUDED_)
#define AFX_TFWKUITESTCFG_H__33520481_4266_11D3_BE2D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkUiTest.h"
#include "Tfwk.h"
#include "TfwkUiSourceId.h"



class TfwkUiTestCfg
{
public:
	/*!
   * Constructs a test.
   * @param pInfo [n] Information about the test.
   */
	TfwkUiTestCfg( const TfwkUnitTestMethodInfo *pInfo,
                 TfwkUiSourceId idSource );

  /*!
   * Destructor.
   */
	virtual ~TfwkUiTestCfg();

  /*!
   * Gets the class.
   * @return Class name.
   */
  CString Class() const;

  /*! 
   * Gets the method.
   * @return Method name.
   */
  CString Method() const;

  /*!
   * Gets the id of the source this test come from.
   * @return Id of the source this test come from.
   */
  TfwkUiSourceId idSource() const;

  /*!
   * Gets the full method name (class::method).
   * @param strFullName [Out] Full method name.
   */
  void FullMethodName( CString &strFullName ) const;

  /*!
   * Gets the test information.
   * @return Test information.
   */
  const TfwkUnitTestMethodInfo &info() const;

  /*! 
   * Increases the number of suite that reference that test.
   */
  void IncSuiteRef();

  /*!
   * Decreases the number of suite that reference that test.
   */
  void DecSuiteRef();

  /*!
   * Is that test used by any suite ?
   * @return \c true if that test is used by some suite.
   */
  bool IsUsed() const;


private:
  /*!
   * Test method info.
   */
  const TfwkUnitTestMethodInfo *_pInfo;

  /*!
   * Number of suite that use that test.
   */
  int _nRefCount;  

  /*!
   * Source id.
   */
  int _idSource;
};



inline CString
TfwkUiTestCfg::Class() const
{
  return info().unitTest().nameTest();
}


inline CString
TfwkUiTestCfg::Method() const
{
  return info().name();
}


inline const TfwkUnitTestMethodInfo &
TfwkUiTestCfg::info() const
{
  ASSERT( _pInfo );
  return *_pInfo;
}


inline void 
TfwkUiTestCfg::IncSuiteRef()
{
  ++_nRefCount;
}


inline void 
TfwkUiTestCfg::DecSuiteRef()
{
  ASSERT( _nRefCount > 0 );
  --_nRefCount;
}


inline bool 
TfwkUiTestCfg::IsUsed() const
{
  return _nRefCount > 0;
}


inline void 
TfwkUiTestCfg::FullMethodName( CString &strFullName ) const
{
  strFullName.Format( "%s::%s", 
                      (LPCTSTR)Class(), 
                      (LPCTSTR)Method() );
}

inline TfwkUiSourceId 
TfwkUiTestCfg::idSource() const
{
  return _idSource;
}

#endif // !defined(AFX_TFWKUITESTCFG_H__33520481_4266_11D3_BE2D_0040053AD648__INCLUDED_)
