// TfwkUnitTestInfo.h: interface for the TfwkUnitTestInfo class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUNITTESTINFO_H__43290697_4193_11D3_BE29_0040053AD648__INCLUDED_)
#define AFX_TFWKUNITTESTINFO_H__43290697_4193_11D3_BE29_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkClass.h"
#include "TfwkArrayT.h"
#include "TfwkArrayItT.h"

class TfwkUnitTestMethodInfo;

typedef TfwkArrayT<const TfwkUnitTestMethodInfo *>::TfwkConstIt TfwkMethodInfoIt;


/*! \class TfwkUnitTestInfo
 * \include "TfwkUnitTestInfo.h"
 * \brief This class holds information about a Unit Test.
 */
class TFWK_UI_EXPORT TfwkUnitTestInfo  
{
public:
  /*!
   * Type of method information array.
   */
  typedef TfwkArrayT<const TfwkUnitTestMethodInfo *> TfwkMethodInfoArray;

  /*! 
   * Constructs a Unit Test Information.
   * @param clsidUnitTest Unit Test class id.
   */
	TfwkUnitTestInfo( const TfwkClassId &clsidUnitTest );

  /*!
   * Destructor.
   */
  virtual ~TfwkUnitTestInfo() {}

  /*!
   * Gets the class id of the Unit Test.
   * @return Class id of the Unit Test.
   */
  const TfwkClassId &clsidTest() const;

  /*!
   * Gets the name of the Unit Test.
   * @return Name of the Unit Test. Never \c NULL.
   */
  const char *nameTest() const;

  /*!
   * Gests an iterator on the method info.
   * @return Iterator on the method info.
   */
  TfwkMethodInfoIt itMethod() const;

  /*!
   * Gets a pointer on the first Unit Test information.
   * @return Pointer on the first Unit Test information. May be \c NULL.
   */
  static const TfwkUnitTestInfo *firstUnitTest();

  /*!
   * Gets a pointer on the next Unit Test information.
   * @return Pointer on the next Unit Test information. May be \c NULL.
   */
  const TfwkUnitTestInfo *nextUnitTest() const;

protected:
  /*!
   * Prepares method info array.
   *
   * This method is called when trying to retreive an iterator since we
   * can not rely on the static data order of initialization (the array
   * may not be initialized when this object is constructed).
   */
  virtual void prepareMethodInfo() =0;

  /*!
   * Array of pointer on the method info (initialized by subclass).
   */
  TfwkMethodInfoArray _aMethodInfo;

private:
  /*!
   * Unit test class id.
   */
  const TfwkClassId &_clsidUnitTest;

  /*!
   * Pointer on the first Unit Test information.
   */
  static const TfwkUnitTestInfo *_pFirstUnitTest;

  /*!
   * Pointer on the next Unit Test information.
   */
  const TfwkUnitTestInfo *_pNextUnitTest;
};


// Inline : TfwkUnitTestInfo
// -------------------------

inline const char *
TfwkUnitTestInfo::nameTest() const
{
  return _clsidUnitTest.nameClass();
}


inline const TfwkUnitTestInfo *
TfwkUnitTestInfo::firstUnitTest()
{
  return _pFirstUnitTest;
}


inline const TfwkUnitTestInfo *
TfwkUnitTestInfo::nextUnitTest() const
{
  return _pNextUnitTest;
}


inline TfwkMethodInfoIt 
TfwkUnitTestInfo::itMethod() const
{
  ((TfwkUnitTestInfo *)this)->prepareMethodInfo();    // mutable call...
  return TfwkMethodInfoIt( _aMethodInfo );
}


inline const TfwkClassId &
TfwkUnitTestInfo::clsidTest() const
{
  return _clsidUnitTest;
}


#endif // !defined(AFX_TFWKUNITTESTINFO_H__43290697_4193_11D3_BE29_0040053AD648__INCLUDED_)
