// TfwkUnitTestInfoT.h: interface for the TfwkUnitTestInfoT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUNITTESTINFOT_H__4329069C_4193_11D3_BE29_0040053AD648__INCLUDED_)
#define AFX_TFWKUNITTESTINFOT_H__4329069C_4193_11D3_BE29_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkUnitTestInfo.h"
//#include "TfwkUnitTestMethodInfoT.h"
#include "TfwkUnitTestMethodInfo.h"


/*! \class TfwkUnitTestInfoT<UnitTest>
 * \include "TfwkUnitTestInfoT.h"
 * \brief This class holds information about a given unit test.
 */
template<class UnitTest>
class TFWK_UI_EXPORT TfwkUnitTestInfoT : public TfwkUnitTestInfo
{
public:

  /*! 
   * Type of method information.
   */
  typedef TfwkUnitTestMethodInfo TypeMethodInfo;

  /*! 
   * Constructs a Unit Test Information.
   * @param clsidUnitTest Unit Test class id.
   * @param aMethodInfo Array of method information.
   */
	TfwkUnitTestInfoT( const TfwkClassId &clsidUnitTest,
                     TypeMethodInfo *aMethodInfo );

  /*!
   * Destructor.
   */
  virtual ~TfwkUnitTestInfoT() {}

protected:
  /*!
   * Prepares method info array.
   *
   * This method is called when trying to retreive an iterator since we
   * can not rely on the static data order of initialization (the array
   * may not be initialized when this object is constructed).
   */
  virtual void prepareMethodInfo();

private:
  /*!
   * Method information c-array (static array).
   */
  TypeMethodInfo *_pMethodInfo;
};




// Template implemenation : TfwkUnitTestInfoT<UnitTest>
// ----------------------------------------------------

template<class UnitTest>
TfwkUnitTestInfoT<UnitTest>::TfwkUnitTestInfoT( const TfwkClassId &clsidUnitTest,
                                                TypeMethodInfo *aMethodInfo ) :
    TfwkUnitTestInfo( clsidUnitTest ),
    _pMethodInfo( aMethodInfo )
{
}


template<class UnitTest>
void
TfwkUnitTestInfoT<UnitTest>::prepareMethodInfo()
{
  if ( _pMethodInfo )
  {
    int nIndex;
    for ( nIndex =0; _pMethodInfo[ nIndex ].name() != NULL; ++nIndex )
    {
      TfwkUnitTestMethodInfo *pMethodInfo = &_pMethodInfo[ nIndex ];
      pMethodInfo->setUnitTestInfo( this );
      _aMethodInfo.add( pMethodInfo );
    }
    _pMethodInfo =NULL;   // indicates that the array has already been prepared !
  }
}


#endif // !defined(AFX_TFWKUNITTESTINFOT_H__4329069C_4193_11D3_BE29_0040053AD648__INCLUDED_)
