// TfwkUnitTestMethodInfo.h: interface for the TfwkUnitTestMethodInfo class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUNITTESTMETHODINFO_H__43290696_4193_11D3_BE29_0040053AD648__INCLUDED_)
#define AFX_TFWKUNITTESTMETHODINFO_H__43290696_4193_11D3_BE29_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkCommon.h"
class TfwkUnitTest;
class TfwkUnitTestInfo;


class TFWK_UI_EXPORT TfwkUnitTestMethodInfo  
{
public:
  /*!
   * Type of a method.
   */
  typedef void (TfwkUnitTest::*TypeMethod)();

  /*!
   * Constructs a unit test method information that indicates the end of the list.
   */
  TfwkUnitTestMethodInfo() : _szName( NULL ) {}

  /*!
   * Constructs a unit test method information.
   * 
   * @param szName Name of the method.
   */
	TfwkUnitTestMethodInfo( const char *szName,
                          TypeMethod fnTest );

  /*! 
   * Destructor.
   */
  virtual ~TfwkUnitTestMethodInfo() {}

  /*! 
   * Gets the name of the method.
   * @return Name of the method. If \c NULL then indicates end of method list.
   */
  const char *name() const;

  /*!
   * Gets the unit test information this method is part of.
   * @return Unit tes information this method is part of.
   */
  const TfwkUnitTestInfo &unitTest() const;

  /*!
   * Runs the method on the given object.
   * @param pBaseUnitTest Object on which the method is run. Must not be \c NULL.
   */
  void run( TfwkUnitTest *pBaseUnitTest ) const;

  /*!
   * Sets the Unit Test information this method is part of 
   * (called by TfwkUnitTestInfoT).
   * @param pUnitTestInfo Pointer on the Unit Test information.
   */
  void setUnitTestInfo( TfwkUnitTestInfo *pUnitTestInfo );

private:
  /*! 
   * Name of the method.
   */
  const char *_szName;

  /*! 
   * Test method.
   */
  TypeMethod _fnTest;

  /*!
   * Unit Test this method is part of.
   */
  TfwkUnitTestInfo *_pUnitTestInfo;
};



// Template implementation : TfwkUnitTestMethodInfo
// -----------------------------------------------------------

inline
TfwkUnitTestMethodInfo::TfwkUnitTestMethodInfo( const char *szName,
                                                TypeMethod fnTest ) :
    _szName( szName ),
    _fnTest( fnTest ),
    _pUnitTestInfo( NULL )
{
}


inline const char *
TfwkUnitTestMethodInfo::name() const
{
  return _szName;
}


inline void 
TfwkUnitTestMethodInfo::run( TfwkUnitTest *pBaseUnitTest ) const
{
  (pBaseUnitTest->*_fnTest)();
}


inline void 
TfwkUnitTestMethodInfo::setUnitTestInfo( TfwkUnitTestInfo *pUnitTestInfo )
{
  _pUnitTestInfo = pUnitTestInfo;
}


inline const TfwkUnitTestInfo &
TfwkUnitTestMethodInfo::unitTest() const
{
  TFWKASSERT( _pUnitTestInfo );
  return *_pUnitTestInfo;
}


#endif // !defined(AFX_TFWKUNITTESTMETHODINFO_H__43290696_4193_11D3_BE29_0040053AD648__INCLUDED_)
