// TabModeAnalysis.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "TabModeAnalysis.h"
#include "TtrDlgEnclosure.h"
#include "TtrGlobalSetting.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTabModeAnalysis property page

IMPLEMENT_DYNCREATE(CTabModeAnalysis, CPropertyPage)

CTabModeAnalysis::CTabModeAnalysis() : CPropertyPage(CTabModeAnalysis::IDD)
{
	//{{AFX_DATA_INIT(CTabModeAnalysis)
	m_indentEmptyLine = FALSE;
	m_indentDoubleQuote = FALSE;
	m_indentIndentedLine = FALSE;
	m_indentPreviousLineIsEmpty = FALSE;
	m_indentSeparator = FALSE;
	m_indentScript = FALSE;
	m_strSeparatorChar = _T("");
	_strBeginWith = _T("");
	_bIndentBeginWith = FALSE;
	_strEnclosure = _T("");
	_bEnclosure = FALSE;
	_bLeftMargin = FALSE;
	_nLeftMargin = 0;
	_bNotLowerCaseLine = FALSE;
	//}}AFX_DATA_INIT

  _listEnclosure.addListener( this );
}


CTabModeAnalysis::~CTabModeAnalysis()
{
  _listEnclosure.removeListener( this );
}


void 
CTabModeAnalysis::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTabModeAnalysis)
	DDX_Check(pDX, IDC_EMPTY_LINE, m_indentEmptyLine);
	DDX_Check(pDX, IDC_LINE_INDENTED, m_indentIndentedLine);
	DDX_Check(pDX, IDC_PREVIOUS_LINE_EMPTY, m_indentPreviousLineIsEmpty);
	DDX_Check(pDX, IDC_SEPARATOR_LINE, m_indentSeparator);
	DDX_Check(pDX, IDC_SCRIPT_MODE, m_indentScript);
	DDX_Text(pDX, IDC_SEPARATOR_CHAR, m_strSeparatorChar);
	DDX_Text(pDX, IDC_EDIT_BEGIN_WITH, _strBeginWith);
	DDX_Check(pDX, IDC_INDENT_BEGIN_WITH, _bIndentBeginWith);
	DDX_Text(pDX, IDC_EDIT_ENCLOSURE, _strEnclosure);
	DDX_Check(pDX, IDC_ENCLOSURE, _bEnclosure);
	DDX_Check(pDX, IDC_CHECK_LEFT_MARGIN, _bLeftMargin);
	DDX_Text(pDX, IDC_EDIT_LEFT_MARGIN, _nLeftMargin);
	DDX_Check(pDX, IDC_DOES_NOT_START_WITH_UPPER_CASE, _bNotLowerCaseLine);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTabModeAnalysis, CPropertyPage)
	//{{AFX_MSG_MAP(CTabModeAnalysis)
	ON_BN_CLICKED(IDC_ENCLOSURE_SPECIFY, OnEnclosureSpecify)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void 
CTabModeAnalysis::GetMode( TtrGlobalSetting& mode ) const
{
  mode.setFormatIndentBlankLine( m_indentEmptyLine );
  mode.setFormatIndentIndentedLine( m_indentIndentedLine );
  mode.setFormatIndentPreviousLineIsBlank( m_indentPreviousLineIsEmpty );
  mode.setFormatIndentSeparator( m_indentSeparator );
  mode.setFormatIndentContainDoubleDot( m_indentScript );
  mode.setFormatSeparatorChar( (LPCTSTR)m_strSeparatorChar );
  mode.setFormatIndentBeginWithChar( (LPCTSTR)_strBeginWith );
  mode.setFormatIndentBeginWith( _bIndentBeginWith );
  mode.setFormatEnclosureList( _listEnclosure );
  mode.setFormatUseEnclosure( _bEnclosure );
  mode.setFormatUseLeftMargin( _bLeftMargin );
  mode.setFormatLeftMargin( _nLeftMargin );
  mode.setFormatNotLowerCaseLine( _bNotLowerCaseLine );
}


void 
CTabModeAnalysis::SetMode( const TtrGlobalSetting& mode )
{
  m_indentEmptyLine = mode.getFormatIndentBlankLine();
  m_indentIndentedLine = mode.getFormatIndentIndentedLine();
  m_indentPreviousLineIsEmpty = mode.getFormatIndentPreviousLineIsBlank();
  m_indentSeparator = mode.getFormatIndentSeparator();
  m_indentScript = mode.getFormatIndentContainDoubleDot();
  m_strSeparatorChar = mode.getFormatSeparatorChar();
  _strBeginWith = mode.getFormatIndentBeginWithChar();
  _bIndentBeginWith = mode.getFormatIndentBeginWith();
  _listEnclosure = mode.getFormatEnclosureList();
  _bEnclosure = mode.getFormatUseEnclosure();
  _bLeftMargin = mode.getFormatUseLeftMargin();
  _nLeftMargin = mode.getFormatLeftMargin();
  _bNotLowerCaseLine = mode.getFormatNotLowerCaseLine();
}

/////////////////////////////////////////////////////////////////////////////
// CTabModeAnalysis message handlers

void 
CTabModeAnalysis::OnEnclosureSpecify() 
{
  TtrDlgEnclosure dlg( _listEnclosure );
  if ( dlg.DoModal() == IDOK )
  {
    _listEnclosure = dlg.getEnclosureList();
  }
}


void
CTabModeAnalysis::updateEnclosure()
{
  ItTtrEnclosure *pIt = _listEnclosure.iterator();
  _strEnclosure = _T( "" ); 
  for ( pIt->begin(); !pIt->isEnd(); pIt->next() )
  {
    _strEnclosure += pIt->item()->getOpening() + pIt->item()->getClosure();
    if ( !pIt->isEnd() )
      _strEnclosure += _T( ";" );
  }

  delete pIt;

  if ( GetSafeHwnd() )
    UpdateData( FALSE );
}


void 
CTabModeAnalysis::enclosureAdded( TtrEnclosureList &list, TtrEnclosure &enclosure )
{
  updateEnclosure();
}


void 
CTabModeAnalysis::enclosureRemoved( TtrEnclosureList &list, TtrEnclosure *pEnclosure )
{
  updateEnclosure();
}


void 
CTabModeAnalysis::enclosureListRenewed( TtrEnclosureList &list )
{
  updateEnclosure();
}
