// Generated by Together

#ifndef TTRTEXT_H
#define TTRTEXT_H

#include "TtrTextBlockDef.h"
#include "TtrTextLine.h"

class TtrTextLine;
class EwgFindTextInfo;
class TtrTextModifiedTarget;

class TtrText 
{
public:
  /** Functor use with forEach().
   */
  struct TtrTextForEachFn
  {
    virtual void operator ()( TtrTextLine &line, 
                              SL nLineBlock ) =0;
  };

  
public:
  TtrText();
  virtual ~TtrText();

  /** Get the number of lines.
   * @return Number of lines.
   */
  virtual SL nbLine() const =0;
  
  /** Retreive a line of text.
   * @param nLineNo Line number (starting from 0).
   * @return Reference on the line.
   */
  virtual TtrTextLine &line( SL nLineNo ) =0;

  /** Retreive a line of text.
   * @param nLineNo Line number (starting from 0).
   * @return Reference on the line.
   */
  virtual const TtrTextLine &line( SL nLineNo ) const =0;
  
  /** Delete all the text.
   */
  virtual void clear() =0;

  /** Insert a line of text.
   * @param pLine Pointer on the line to insert.
   * @param nLineNo Insertion position (0 based). -1 to insert at the end of the text.
   */
  void insertLine( TtrTextLine *pLine,
                   SL nLineNo =-1 );
  
 /** Delete a line of text.
   * @param nLineNo Number of the line to delete (starting from 0).
   */
  void deleteLine( SL nLineNo );
  
  /** Indicate that the text has been modified.
   */
  void setModified( bool bModified =true );
  
  /** Set the target for modified message.
   * @param pTarget Pointer on the target that is informed when the text is modified.
   *        May be \c NULL.
   */
  void setModifiedTarget( TtrTextModifiedTarget *pTarget );
  
  /** Test if a line number is valid.
   */
  bool isValidLineNo( SL nLineNo ) const;

  /** Call a function for each line of a block.
   * @param [In] nStart First line of the block.
   * @param [In] nLast Last line of the block.
   * @param [In/Out] fn Functor to call.
   */
  void forEach( const TtrTextBlockDef &defBlock, 
                TtrTextForEachFn &fn );
  /** Indent each line of the block.*/
  void indentBlock( const TtrTextBlockDef &defBlock );       
  
  /** Reset block indentation level to normal.*/
  void restoreBlock( const TtrTextBlockDef &defBlock );       
  
  /** Glue block.*/
  void glueBlock( const TtrTextBlockDef &defBlock );

  /** Find the next occurence of the string.
   * @param nStartLineNo Line the search begin at (not included).
   * @param info Search criteria.
   * @return -1 if no other occurence are found.
   */
  SL findString( SL nStartLineNo,
                 const EwgFindTextInfo &info );

  /** Maximum line length.
   */
  SL maxLineLength() const;

  /** Returns the text name.
   * @return Text name (filename).
   */
  const EString &getName() const;

  /** Sets the text name.
   * @param strName Text name.
   */
  void setName( const EString &strName );

protected:
  /** Insert a line of text.
   * @param pLine Pointer on the line to insert.
   * @param nLineNo Insertion position (0 based). -1 to insert at the end of the text.
   */
  virtual void doInsertLine( TtrTextLine *pLine,
                           SL nLineNo =-1 ) =0;
  
 /** Delete a line of text.
   * @param nLineNo Number of the line to delete (starting from 0).
   */
  virtual void doDeleteLine( SL nLineNo ) =0;

private:
  /**@shapeType DependencyLink*/
  /*#  TtrTextForEachFnIndent lnkUnnamed3*/
  /**@shapeType DependencyLink*/
  /*#  TtrTextForEachFnReset lnkUnnamed2*/
  /**@shapeType DependencyLink*/
  /*#  TtrTextForEachFnGlue lnkUnnamed1*/

  /** Test if a line match the search criteria.
   * @param line Line to check.
   * @param info Search criteria.
   * @param strToFind String searched for. Must be in lower if search is case 
   *        insensitive.
   * @return \c true if the line match the search criteria, otherwise
   *         returns \c false.
   */
  bool lineMatchSearchCriteria( const TtrTextLine &line,
                                const EwgFindTextInfo &info,
                                const EString &strToFind );

  /** Type for the list of text line.
   */
  typedef EPtrDynArrayT<TtrTextLine *> TtrListTextLine;

  /** Pointer on the target that is informed when the text is modified.
   */
  TtrTextModifiedTarget *_pTargetModified;

  /** Maximum line length in characters.
   */
  SL _nMaxLineLength;

  /** Text name.
   */
  EString _strName;
};




// Inline TtrText:
// ---------------

inline SL 
TtrText::maxLineLength() const
{
  return _nMaxLineLength;
}


inline void 
TtrText::setModifiedTarget( TtrTextModifiedTarget *pTarget )
{
  _pTargetModified = pTarget;
}


inline bool
TtrText::isValidLineNo( SL nLineNo ) const
{
  return nLineNo >=0  &&  nLineNo < nbLine();
}






#endif //TTRTEXT_H



