/* - ExtractMethod
   - MergeConditionalTemp <<<new>>>
    if ( mode.m_indentSquareBracket )
      beginSquareBracket = tcFirst == _T( '[' );  // line begin with '['

    if ( mode.m_indentRoundBracket )
      beginRoundBracket = tcFirst == _T( '(' );  // line begin with '('

    [...]
    
    if ( beginSquareBracket  ||
         beginRoundBracket  )

    ==>

    bool bIndentBeginWith = false;
    if ( mode.m_indentSquareBracket )
      bIndentBeginWith = tcFirst == _T( '[' );  // line begin with '['

    if ( mode.m_indentRoundBracket )
      bIndentBeginWith = tcFirst == _T( '(' );  // line begin with '('
   - DecomposeConditionnal
   - RemoveControlFlag
   - ReplaceTempWithQuery
   - ReplaceTempWithQuery
   - ReplaceTempWithQuery
   - ReplaceTempWithQuery
   - ReplaceTempWithQuery
   - RenameTempVariable   <<< new >>>
     quote = ...

     ==>

     bIsWithinEnclosedText = ...
   - ExtractMethod
   - ReplaceTempWithQuery
   - ReduceTempScope      <<< new >>>
     bool bCond = ...;
     {
       bCond = ....
       if ( bCond )....
     }

     ==>

     {
       bool bCond = ...
       if ( bCond ) ...
     }
   - ReduceTempScope      <<< new >>>
   - ReduceTempScope      <<< new >>>
   - InlineTemp
   - InlineTemp
   - ReplaceCondtionnalWithAssigmentToTemp <<<new>>>
     bool bTemp = expression1;
     if ( condition )
       bTemp = expression2;

     ==> 

     bool bTemp = expression1  &&  ( !condition  ||  expression2 )
   - InlineTemp
   - ReduceTempScope      <<< new >>>
   - ReplaceCondtionnalWithAssigmentToTemp <<<new>>>
   - ReplaceTempWithQuery
   - InlineTemp
   - ReformatCode
   - ExtractMethod
   - RenameMethod
   - RenameTemp
   - ExtractMethod
   - ExtractMethod
   # The first step is done. We have a mush clearer code and 
     we can began understand the logic of the method. 
     Let's simplify it a little bit...
   - ExtractFirstLoopBehaviorFromLoop   <<<new>>>
     You have something like:
     bool bIsFirstLoop = true;
     for (...)
     {
       if ( bIsFirstLoop )
       {
          do something for the first loop
       }
       else
       {
         do something else
       }
       always do something 
       bIsFirstLoop = false;
     }

     ==>
     
     do something for the first loop
     always do something 
     bFirst = false;
     for ( ... )
     {
       if ( bIsFirstLoop )
       {
          do something for the first loop
       }
       else
       {
         do something else
       }
       always do something 
     }

     ---
     
     do something for the first loop
     always do something
     bFirst = false;
     for ( ... )
     {
       do something else
       always do something 
     }
   - Inline Temp
   - IntroduceParameterObject (all methods changed)
   - ReplaceTempWithQuery/UseObject (?)
   - RemoveMethod  <<<new>>>
     The method is simple and used in only one or few place, and does
     nothing more than delegating the query to one of the parameter.
   - ??? Embeded logic. ???
   - IntroduceParameterObject
   - ExtractMethod
   - MoveMethod
   - MoveMethod
   - RenameMethod
   - RemoveMethod  <<<new>>>   ### is InlineMethod
   - SeparateQueryFromModifier
   - RemoveUnusedTemp  <<<new>>>
     Some temporary variable are not used anymore, removes them.
   - ??? Simplify logic ???
   - ReplaceTempWithQuery
   - AddParameter
   - ReplaceParameterWithMethod
   - RemoveUnsuedTemp   <<<new>>>
   - ReplaceTempWithQuery
   # The second step is done. We now have a rather nice algorithm,
     which is much more easier to understand and change. During this
     refactoring, we found a few bug. Because we understand the
     logic of the program way better, we can now do some design.
     Here is what I want:
      + Adding new rules to indent a line based on the content of 
        the current or previous line.
      + Adding new enclosing rules to prevent the indentation of
        the line when it is enclosed within bracket...
      I Want to do this without needing to change the existing code,
      so we need to introduce some "extension" mechanisms.
   - ReplaceConditionnalWithPolymorphism
     Note that it is a small variant, instead of applying it to a
     switch, it is applied to a series of if statement which
     test a variable to be taken in account.
       if ( ( mode & xxx)  && cond1 )  ||
          ( ( mode & yyy)  && cond2 ) ....
       We make a super-class with a virtual method for the test.
       We composed the sub-class which implemented each one of the
       test using a composite pattern.
     - MoveMethod
     - RemoveMethod
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     - MoveMethod 
     # With the new design, we can now easilly a new rule based on the
       content of the current or the previous line to indent the current line.
       Let's works on the text enclosure now.
     - SubstituteMethod
       You have another method that does the same jobs. Use it.
       - required: BugFix: Square & Round bracket grouping.... 
         Fixed bugs detected while making Unit Test.
   - IntroduceParameterObject
   - ReplaceParameterWithQuery
   - RemoveClass
   - MoveUpClassScope  <<<new>>>
   - ReplaceParameterWithQuery
   - ExtractClass
   - MoveMethod
   - RemoveMethod
   - RemoveMethod
   - SubstituteAlgorithm
   - InlineMethod
   - RenameMemberVariable
   - ExtractSuperClass
   - SubstituteAlgorithm
 */

/* RuleContext
 * Weirds thing may happen if current line is 0...
 * Should implements a TtrTextLineNull using the NullObject pattern.
 * Should also study if it may be possible to use this object as a caching proxy
 * for the text. Caching would included queries such as line.isEos()... which
 * are time consumming. On another hand, this object is rather volatile and the
 * cache would not be able to carry on from one reformatting to another one.
 */

#include "StdAfx.h"
#include "TtrTextFormaterV0.h"
#include "TtrGlobalSetting.h"
#include "TtrEnclosure.h"

// ---------------------------------------------------
// Class RuleContext
// ---------------------------------------------------

class RuleContext
{
public:
  RuleContext( TtrText &text, 
               SL nCurLineNo,
               TtrFormatSetting &setting );

  TtrTextLine &line() const;
  TtrTextLine &linePrev() const;
  TtrFormatSetting &setting() const;

private:  
  TtrText &_text;
  TtrTextLine *_pLine;
  TtrTextLine *_pLinePrev;
  TtrFormatSetting &_setting;
  SL _nCurLineNo;
};


RuleContext::RuleContext( TtrText &text, 
                          SL nCurLineNo,
                          TtrFormatSetting &setting ) :
    _text( text ),
    _nCurLineNo( nCurLineNo ),
    _setting( setting )
{
  _pLine = &text.line( _nCurLineNo );
  _pLinePrev = &text.line( _nCurLineNo -1 ); 
}


TtrTextLine &
RuleContext::line() const
{
  return *_pLine;
}


TtrTextLine &
RuleContext::linePrev() const
{
  return *_pLinePrev;
}


TtrFormatSetting &
RuleContext::setting() const
{
  return _setting;
}

// ---------------------------------------------------
// Class RuleIndentation
// ---------------------------------------------------
class RuleIndentation
{
public:
  RuleIndentation() {}
  virtual ~RuleIndentation() {}

  /*!
   * Applies the rule on the specified context.
   * @param line Current line.
   * @param linePrev Previous line.
   * @param mode Analysis settings.
   * @return \c true if the line must be indented,
   *         otherwise returns \c false.
   */
  virtual bool applyRule( const RuleContext &ctx ) =0;
};


// ---------------------------------------------------
// Class RuleIndentationComposite
// ---------------------------------------------------
class RuleIndentationComposite : public RuleIndentation
{
public:
  RuleIndentationComposite() {}
  virtual ~RuleIndentationComposite() {}
  
  /*!
   * Applies the rule on the specified context.
   * Calls the applyRule() method on each added rule. If one of the rule returns
   * \c true then return \c true.
   * @param line Current line.
   * @param linePrev Previous line.
   * @param mode Analysis settings.
   * @return \c true if one of the rule that has been added return \c true.
   */
  virtual bool applyRule( const RuleContext &ctx );

  /*!
   * Adds a rule to the composite.
   * @param pRule Pointer on the rule to add. It will be destroyed by the compisite.
   */
  void add( RuleIndentation *pRule );

  /*! Removes a rule from the composite.
   * @param pRule Pointer on the rule to remove.
   */
  void remove( RuleIndentation *pRule );

private:
  typedef EPtrDynArrayT<RuleIndentation *> ArrayRule;
  ArrayRule _apRule;
};


// ---------------------------------------------------
// Class RuleIndentationIsBlank
// ---------------------------------------------------
class RuleIndentationIsBlank : public RuleIndentation
{
public:
  RuleIndentationIsBlank() {}
  virtual ~RuleIndentationIsBlank() {}

  virtual bool applyRule( const RuleContext &ctx  )
  {
    return ctx.line().isBlank();
  }
};


// ---------------------------------------------------
// Class RuleIndentationIsBlankPrev
// ---------------------------------------------------
class RuleIndentationIsBlankPrev : public RuleIndentation
{
public:
  RuleIndentationIsBlankPrev() {}
  virtual ~RuleIndentationIsBlankPrev() {}

  virtual bool applyRule( const RuleContext &ctx  )
  {
    return ctx.linePrev().isBlank();
  }
};


// ---------------------------------------------------
// Class RuleIndentationIsIndented
// ---------------------------------------------------
class RuleIndentationIsIndented : public RuleIndentation
{
public:
  RuleIndentationIsIndented( SL nLeftMargin ) :
      _nLeftMargin( nLeftMargin ) {}
  virtual ~RuleIndentationIsIndented() {}

  virtual bool applyRule( const RuleContext &ctx  )
  {
    return ctx.line().indentCount() > _nLeftMargin;
  }

private:
  SL _nLeftMargin;
};


// ---------------------------------------------------
// Class RuleIndentationBeginWith
// ---------------------------------------------------
class RuleIndentationBeginWith : public RuleIndentation
{
public:
  RuleIndentationBeginWith( const EString &strBeginChar ) :
      _strBeginChar( strBeginChar )
  {
  }

  virtual ~RuleIndentationBeginWith() {}

  virtual bool applyRule( const RuleContext &ctx  )
  {
    return _strBeginChar.contains( ctx.line().firstNonWhiteChar() );
  }

private:
  EString _strBeginChar;
};


// ---------------------------------------------------
// Class RuleIndentationIsScript
// ---------------------------------------------------
class RuleIndentationIsScript : public RuleIndentation
{
public:
  RuleIndentationIsScript() {}
  virtual ~RuleIndentationIsScript() {}

  virtual bool applyRule(  const RuleContext &ctx  )
  {
    if ( ctx.line().contains( _T( ':' ) ) )
    { // check if ':' is the last character on the line, or if there are more than 1
      return ( ctx.line().at( ctx.line().firstTrailingSpacePos() -1 ) != _T( ':' ) )  ||
             ( ctx.line().pos( _T( ':' ) ) != ctx.line().posLast( _T( ':' ) ) );
    }

    return false;
  }
};


// ---------------------------------------------------
// Class RuleIndentationIsSeparator
// ---------------------------------------------------
class RuleIndentationIsSeparator : public RuleIndentation
{
public:
  RuleIndentationIsSeparator() {}
  virtual ~RuleIndentationIsSeparator() {}

  virtual bool applyRule(  const RuleContext &ctx  )
  {
    return ctx.line().isSeparatorLine( ctx.setting().getFormatSeparatorChar() )  ||
           ctx.linePrev().isSeparatorLine( ctx.setting().getFormatSeparatorChar() );
  }
};


// ---------------------------------------------------
// Class RuleIndentationIsEosPrev
// ---------------------------------------------------
class RuleIndentationIsEosPrev : public RuleIndentation
{
public:
  RuleIndentationIsEosPrev() {}
  virtual ~RuleIndentationIsEosPrev() {}

  virtual bool applyRule( const RuleContext &ctx  )
  {
    SL nPosLastChar = ctx.linePrev().firstTrailingSpacePos() -1;
    return ( nPosLastChar >= 0 )  &&
           ctx.setting().getFormatModeEosChar().contains( 
              ctx.linePrev().at( nPosLastChar ) );
  }
};


// ---------------------------------------------------
// Class RuleIndentationRightMargin
// ---------------------------------------------------

class RuleIndentationRightMargin : public RuleIndentation
{
public:
  RuleIndentationRightMargin( SL nRightMargin,
                              bool bIndentBlankLine ) :
      _nRightMargin( nRightMargin ),
      _bIndentBlankLine( bIndentBlankLine )
  {
    initWordLetter();
  }

  virtual ~RuleIndentationRightMargin() {}

  virtual bool applyRule( const RuleContext &ctx  )
  {
    SL nFirstWordLength = findFirstWordLength( ctx );
    if ( nFirstWordLength == 0 )    // blank line ? 
      return _bIndentBlankLine;         // Returns "indent blank line" rule status.

    // The word can fit on the previous line?
    return (ctx.linePrev().length() + nFirstWordLength +1 ) <= _nRightMargin;
  }

  static SL findFirstWordLength( const RuleContext &ctx )
  {
    EString strWord;
    SL nWordStart = ctx.line().countHeadingSpace();
    SL nWordEnd;
    for ( nWordEnd = nWordStart; nWordEnd < ctx.line().length(); ++nWordEnd )
      if ( !_vbIsWordLetter[ ctx.line().at( nWordEnd ) ] )
        break;

    return nWordEnd - nWordStart;
  }

  void initWordLetter()
  {
    if (_bWordLetterInitialized )
      return;

    int nIdx;
//    const EString strWordLetter( 
//        "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
//        "0123456789':,;" 
//        );
    for ( nIdx =0; nIdx < 256; ++nIdx )
      _vbIsWordLetter[ nIdx ] = nIdx > 32;
//      _vbIsWordLetter[ nIdx ] = strWordLetter.contains( nIdx );

    _bWordLetterInitialized = false;
  }

private:
  static bool _vbIsWordLetter[256];
  static bool _bWordLetterInitialized;
  int _nRightMargin;
  bool _bIndentBlankLine; 
};

bool RuleIndentationRightMargin::_vbIsWordLetter[256];
bool RuleIndentationRightMargin::_bWordLetterInitialized =false;

// Implementation class RuleIndentationComposite
// ---------------------------------------------------

bool
RuleIndentationComposite::applyRule( const RuleContext &ctx  )
{
  ArrayRule::EIterator it( _apRule );
  for ( it.begin(); !it.isEnd(); it.next() )
    if ( it.item()->applyRule( ctx ) )
      return true;
  return false;
}


void 
RuleIndentationComposite::add( RuleIndentation *pRule )
{
  if ( pRule )
    _apRule.append( pRule );
}


void 
RuleIndentationComposite::remove( RuleIndentation *pRule )
{
  if ( pRule )
  {
    SL nIdx = _apRule.find( pRule );
    if ( nIdx >= 0)
      _apRule.remove( nIdx );
  }
}


// ---------------------------------------------------
// Class RuleEnclosed
// ---------------------------------------------------
class RuleEnclosed
{
public:
  RuleEnclosed() {}
  virtual ~RuleEnclosed() {}

  virtual void processLine( const RuleContext &ctx ) =0;

  virtual bool isInside() const =0;
};


// ---------------------------------------------------
// Class RuleEnclosedComposite
// ---------------------------------------------------
class RuleEnclosedComposite : public RuleEnclosed
{
public:
  RuleEnclosedComposite() {}
  virtual ~RuleEnclosedComposite() {}

  virtual void processLine( const RuleContext &ctx );

  virtual bool isInside() const;

  void add( RuleEnclosed *pRule );
  void remove( RuleEnclosed *pRule );

private:
  typedef EPtrDynArrayT<RuleEnclosed *> ArrayRule;
  ArrayRule _apRule;
};



void
RuleEnclosedComposite::processLine( const RuleContext &ctx  )
{
  ArrayRule::EIterator it( _apRule );
  for ( it.begin(); !it.isEnd(); it.next() )
    it.item()->processLine( ctx );
}


bool 
RuleEnclosedComposite::isInside() const
{
  ArrayRule::EIterator it( _apRule );
  for ( it.begin(); !it.isEnd(); it.next() )
    if ( it.item()->isInside() )
      return true;
  return false;
}


void 
RuleEnclosedComposite::add( RuleEnclosed *pRule )
{
  if ( pRule )
    _apRule.append( pRule );
}


void 
RuleEnclosedComposite::remove( RuleEnclosed *pRule )
{
  if ( pRule )
  {
    SL nIdx = _apRule.find( pRule );
    if ( nIdx >= 0)
      _apRule.remove( nIdx );
  }
}


// ---------------------------------------------------
// Class RuleEnclosedString
// ---------------------------------------------------
class RuleEnclosedString : public RuleEnclosed
{
public:
  RuleEnclosedString( bool bIsInside,
                      const TtrEnclosure &enclosure ) :
      _bIsInside( bIsInside ),
      _enclosure( enclosure ) {}
  virtual ~RuleEnclosedString() {}

  virtual void processLine( const RuleContext &ctx )
  {
    SL nIdx =0;
    SL nLength = ctx.linePrev().length();

    while ( nIdx < nLength )
    {
      const EString &strToMatch = _bIsInside ? _enclosure.getClosure() : 
                                               _enclosure.getOpening();
      nIdx = ctx.linePrev().pos( nIdx, strToMatch );
      if ( nIdx >= 0 )
      {
        _bIsInside = !_bIsInside;
        ++nIdx;
      }
      else
        break;
    }
  }

  virtual bool isInside() const
  {
    return _bIsInside;
  }
private:
  bool _bIsInside;
  const TtrEnclosure &_enclosure;
};


// ---------------------------------------------------
// Class RuleEnclosed
// ---------------------------------------------------
/*! \brief Rule to prevent the indentation if the line starts with a lower case.
 */
class RuleEnclosedNotLowerCaseLine : public RuleEnclosed
{
public:
  RuleEnclosedNotLowerCaseLine() :
      _bFirstIsLower( false ) {}
  virtual ~RuleEnclosedNotLowerCaseLine() {}

  virtual void processLine( const RuleContext &ctx );

  virtual bool isInside() const;
private:
  bool _bFirstIsLower;
};



void 
RuleEnclosedNotLowerCaseLine::processLine( const RuleContext &ctx )
{
  _bFirstIsLower = ::islower( ctx.line().firstNonWhiteChar() )  &&
                    ctx.line().firstNonWhiteChar() != 0;
}


bool 
RuleEnclosedNotLowerCaseLine::isInside() const
{
  return _bFirstIsLower;
}


// ---------------------------------------------------
// Class TtrTextFormaterV0
// ---------------------------------------------------


void 
TtrTextFormaterV0::doFormatBlock( TtrTextBlock &block,
                                  TtrFormatSetting &setting )
{
  TtrText &text = block.text();
  const TtrTextBlockDef &range = block.range();

  if ( !block.isValid() )
    return;

  text.setModified();

  setProgressRange( range.firstLine(),
                    range.lastLine() );
  startProgress();

  RuleIndentationComposite ruleIndentation;
  setupIndentationRules( setting, ruleIndentation );

  RuleEnclosedComposite ruleEnclosed;
  setupEnclosedRules( setting, ruleEnclosed );

  // Specific for the first line
  SL nLineNo = range.firstLine();
//  text.line( nLineNo ).setIndentLevel( 
//      ruleEnclosed.isInside( RuleContext( text, nLineNo, mode ) ) ? -1 : 1 );
  text.line( nLineNo ).setIndentLevel( 1 );

  // For the other lines
  for ( ++nLineNo; nLineNo <= range.lastLine(); ++nLineNo )
  {
    TtrTextLine &line = text.line( nLineNo );

    if ( (nLineNo & 127) == 0 )  // show progress in status bar
      setProgress( nLineNo );

    RuleContext ctx( text, nLineNo, setting );
    ruleEnclosed.processLine( ctx );
    if ( !ruleEnclosed.isInside()  &&  
         ruleIndentation.applyRule( ctx ) )
      line.setIndentLevel( 1 );   // set flag
    else
      line.setIndentLevel( -1 );

  }

  stopProgress();
}


void 
TtrTextFormaterV0::setupIndentationRules( TtrFormatSetting &setting, 
                                          RuleIndentationComposite &ruleIndentation )
{
  if ( setting.getFormatIndentBlankLine() )
    ruleIndentation.add( new RuleIndentationIsBlank() );
  if ( setting.getFormatIndentIndentedLine() )
    ruleIndentation.add( 
        new RuleIndentationIsIndented( 
            setting.getFormatUseLeftMargin() ? setting.getFormatLeftMargin() : 0 ) );
  if ( setting.getFormatIndentContainDoubleDot() )
    ruleIndentation.add( new RuleIndentationIsScript() );
  if ( setting.getFormatIndentSeparator() )
    ruleIndentation.add( new RuleIndentationIsSeparator() );
  if ( setting.getFormatIndentPreviousLineIsBlank() )
    ruleIndentation.add( new RuleIndentationIsBlankPrev() );
  if ( setting.getFormatModeEos() )
    ruleIndentation.add( new RuleIndentationIsEosPrev() );
  
  EString strBeginChar;
  if ( setting.getFormatIndentBeginWith() )
    strBeginChar.append( setting.getFormatIndentBeginWithChar() );

  if ( !strBeginChar.isNull() )
    ruleIndentation.add( new RuleIndentationBeginWith( strBeginChar ) );

  if ( setting.getFormatUseRightMargin() )
    ruleIndentation.add( 
        new RuleIndentationRightMargin( setting.getFormatRightMargin(),
                                        setting.getFormatIndentBlankLine() ) );
}


void 
TtrTextFormaterV0::setupEnclosedRules( TtrFormatSetting &setting, 
                                       RuleEnclosedComposite &ruleEnclosed )
{
  if ( setting.getFormatUseEnclosure() )
  {
    EAutoPtrT<ItTtrEnclosure> pItEnclosure( setting.getFormatEnclosureList().iterator() );
    for ( pItEnclosure->begin(); !pItEnclosure->isEnd(); pItEnclosure->next() )
    {
      ruleEnclosed.add( new RuleEnclosedString( false, *pItEnclosure->item() ) );
    }
  }

  if ( setting.getFormatNotLowerCaseLine() )
    ruleEnclosed.add( new RuleEnclosedNotLowerCaseLine() );
}
