// TtrTextWriterDirectorFsm.cpp: implementation of the TtrTextWriterDirectorFsm class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TtrTextWriterDirectorFsmState.h"
#include "TtrTextWriterDirectorFsm.h"
#include "TtrTextWriter.h"
#include "TtrTextWriterBuilder.h"
#include "TtrWriterSetting.h"
#include "TtrFormatSetting.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// ////////////////////////////////////////////////////////////////////
// TtrTextWriterDirectorFsm
// ////////////////////////////////////////////////////////////////////

TtrTextWriterDirectorFsm::TtrTextWriterDirectorFsm() :
    _pState( &TtrTextWriterDirectorFsmStateInitial::instance() ),
    _pBuilder( NULL )
{
}


TtrTextWriterDirectorFsm::~TtrTextWriterDirectorFsm()
{
}


void
TtrTextWriterDirectorFsm::lineIsBlank()
{
  _pState->lineIsBlank( *this );
}


void
TtrTextWriterDirectorFsm::lineIsIndented()
{
  _pState->lineIsIndented( *this );
}


void
TtrTextWriterDirectorFsm::lineIsGlued()
{
  _pState->lineIsGlued( *this );
}


void
TtrTextWriterDirectorFsm::lineIsOriginal()
{
  _pState->lineIsOriginal( *this );
}


void
TtrTextWriterDirectorFsm::lineIsSeparator()
{
  _pState->lineIsSeparator( *this );
}


void
TtrTextWriterDirectorFsm::endOfText()
{
  _pState->endOfText( *this );
}


void 
TtrTextWriterDirectorFsm::changeState( TtrTextWriterDirectorFsmState &state )
{
  TtrTextWriterDirectorFsmState &stateOld = *_pState;
  _pState = &state;
  stateOld.exitState( *this );
  
  _pState->enterState( *this );
}


void 
TtrTextWriterDirectorFsm::writeText( TtrWriterSetting &setting, 
                                      TtrFormatSetting &settingFormat, 
                                      TtrText &text,           
                                      ETextStream &s,
                                      EwgProgressInfo &progress )
{
  _pSetting = &setting;

  EASSERT( _pBuilder );
  TtrTextWriterBuilder &builder = *_pBuilder;
  builder.setStream( s );

  EASSERT( setting.getWriterType() == wstpV0 );

  progress.setProgressRange( 0, text.nbLine() );
  progress.startProgress();

  builder.buildStart( setting );

  changeState( TtrTextWriterDirectorFsmStateInitial::instance() );

  for ( SL nLineNo =0; nLineNo < text.nbLine(); ++nLineNo )
  {
    const TtrTextLine &line = text.line( nLineNo );
    _pLine = &line;

    if ( line.indentLevel() < 0 )
      lineIsGlued();
    else if ( line.isBlank() )
      lineIsBlank();
    else if ( line.isSeparatorLine( settingFormat.getFormatSeparatorChar() ) )
      lineIsSeparator();
    else if ( line.indentLevel() > 0 )
      lineIsIndented();
    else
      lineIsOriginal();
  }

  endOfText();
  builder.buildEnd( setting );

  progress.stopProgress();
}
