// TtrTextWriterDirectorFsm.h: interface for the TtrTextWriterDirectorFsm class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRTEXTWRITTERDIRECTORFSM_H__722B3947_941D_4F14_B0EC_DAF126FEBEAD__INCLUDED_)
#define AFX_TTRTEXTWRITTERDIRECTORFSM_H__722B3947_941D_4F14_B0EC_DAF126FEBEAD__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TtrTextLine;
class TtrTextWriterBuilder;
class TtrTextWriterDirectorFsm;
class TtrTextWriterDirectorFsmState;
class TtrWriterSetting;
class EwgProgressInfo;
class TtrFormatSetting;
class TtrText;


// ////////////////////////////////////////////////////////////////////
// TtrTextWriterDirectorFsm
// ////////////////////////////////////////////////////////////////////

/*!
 * \brief This class implements a Finite StateMachine to write the text.
 */
class TtrTextWriterDirectorFsm
{
  friend class TtrTextWriterDirectorFsmState;
public:
	TtrTextWriterDirectorFsm();
	virtual ~TtrTextWriterDirectorFsm();


  void setBuilder( TtrTextWriterBuilder &builder );

  virtual void writeText( TtrWriterSetting &setting, 
                          TtrFormatSetting &settingFormat, 
                          TtrText &text,           
                          ETextStream &s,
                          EwgProgressInfo &progress );


  const TtrTextLine &getCurrentLine() const;
  TtrTextWriterBuilder &getBuilder() const;
  TtrWriterSetting &setting() const;

  virtual void lineIsBlank();
  virtual void lineIsIndented();
  virtual void lineIsGlued();
  virtual void lineIsOriginal();
  virtual void lineIsSeparator();
  virtual void endOfText();

private:
  void changeState( TtrTextWriterDirectorFsmState &state );

/** @supplierCardinality 1*/
  TtrWriterSetting *_pSetting;
/** @supplierCardinality 1*/
  TtrTextWriterDirectorFsmState *_pState;
  const TtrTextLine *_pLine;
/** @supplierCardinality 1*/
  TtrTextWriterBuilder *_pBuilder;
};




// implemention of TtrTextWriterDirectorFsm 
// ////////////////////////////////////////////////////////////////////
inline const TtrTextLine &
TtrTextWriterDirectorFsm::getCurrentLine() const
{
  return *_pLine;
}


inline TtrTextWriterBuilder &
TtrTextWriterDirectorFsm::getBuilder() const
{
  return *_pBuilder;
}


inline void 
TtrTextWriterDirectorFsm::setBuilder( TtrTextWriterBuilder &builder )
{
  _pBuilder = &builder;
}


inline TtrWriterSetting &
TtrTextWriterDirectorFsm::setting() const
{
  return *_pSetting;
}



#endif // !defined(AFX_TTRTEXTWRITTERDIRECTORFSM_H__722B3947_941D_4F14_B0EC_DAF126FEBEAD__INCLUDED_)
