// TtrTextWriterDirectorFsmState.cpp: implementation of the TtrTextWriterDirectorFsmState class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TtrTextWriterDirectorFsmState.h"
#include "TtrTextWriterBuilder.h"
#include "TtrTextLine.h"
#include "TtrWriterSetting.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#undef TTRSTATE
#define TTRSTATE( state ) \
    changeState( fsm,TtrTextWriterDirectorFsmState##state::instance() )

// ////////////////////////////////////////////////////////////////////
// TtrTextWriterDirectorFsmStateInitial
// ////////////////////////////////////////////////////////////////////

TtrTextWriterDirectorFsmStateInitial &
TtrTextWriterDirectorFsmStateInitial::instance()
{
  static TtrTextWriterDirectorFsmStateInitial state;
  return state;
}


void 
TtrTextWriterDirectorFsmStateInitial::lineIsBlank( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( BlankBlock );
}


void 
TtrTextWriterDirectorFsmStateInitial::lineIsIndented( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateInitial::lineIsGlued( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateInitial::lineIsOriginal( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateInitial::lineIsSeparator( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Separator );
}


void 
TtrTextWriterDirectorFsmStateInitial::endOfText( TtrTextWriterDirectorFsm &fsm )
{
}




// ////////////////////////////////////////////////////////////////////
// TtrTextWriterDirectorFsmStateBlankBlock
// ////////////////////////////////////////////////////////////////////

TtrTextWriterDirectorFsmStateBlankBlock &
TtrTextWriterDirectorFsmStateBlankBlock::instance()
{
  static TtrTextWriterDirectorFsmStateBlankBlock state;
  return state;
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::enterState( TtrTextWriterDirectorFsm &fsm )
{
  _nBlankLineCount = 1;
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::exitState( TtrTextWriterDirectorFsm &fsm )
{
  fsm.getBuilder().buildBlankBlock( _nBlankLineCount );
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::lineIsBlank( TtrTextWriterDirectorFsm &fsm )
{
  ++_nBlankLineCount;
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::lineIsIndented( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::lineIsGlued( TtrTextWriterDirectorFsm &fsm )
{
  --_nBlankLineCount;
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::lineIsOriginal( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::lineIsSeparator( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Separator );
}


void 
TtrTextWriterDirectorFsmStateBlankBlock::endOfText( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Initial );
}




// ////////////////////////////////////////////////////////////////////
// TtrTextWriterDirectorFsmStateParagraph
// ////////////////////////////////////////////////////////////////////

TtrTextWriterDirectorFsmStateParagraph &
TtrTextWriterDirectorFsmStateParagraph::instance()
{
  static TtrTextWriterDirectorFsmStateParagraph state;
  return state;
}


void 
TtrTextWriterDirectorFsmStateParagraph::enterState( TtrTextWriterDirectorFsm &fsm )
{
  _nParaType = (fsm.getCurrentLine().indentLevel() == 0) ? ptpOriginal :    
                                                           ptpIndented; 
  _strParagraph = fsm.getCurrentLine();
  _strParagraph.trimSpaces();
  _strOriginal = fsm.getCurrentLine().left( fsm.getCurrentLine().countHeadingSpace() );
}


void 
TtrTextWriterDirectorFsmStateParagraph::exitState( TtrTextWriterDirectorFsm &fsm )
{
  if ( _nParaType == ptpIndented )
    fsm.getBuilder().buildParagraph( _strOriginal, _strParagraph );
  else
    fsm.getBuilder().buildOriginal( _strOriginal + _strParagraph );
}


void 
TtrTextWriterDirectorFsmStateParagraph::lineIsBlank( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( BlankBlock );
}


void 
TtrTextWriterDirectorFsmStateParagraph::lineIsIndented( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateParagraph::lineIsGlued( TtrTextWriterDirectorFsm &fsm )
{
  EString strLine( fsm.getCurrentLine() );
  strLine.trimSpaces();
  
  if ( fsm.setting().getWriterRemoveEndingHyphen()  &&
       _strParagraph.lastChar() == '-' )
  {
    if ( !_strParagraph.isNull() )
      _strParagraph.truncate( _strParagraph.length() -1 );
  }
  else
    _strParagraph += ' ';
  
  _strParagraph += strLine;
}


void 
TtrTextWriterDirectorFsmStateParagraph::lineIsOriginal( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateParagraph::lineIsSeparator( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Separator );
}


void 
TtrTextWriterDirectorFsmStateParagraph::endOfText( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Initial );
}




// ////////////////////////////////////////////////////////////////////
// TtrTextWriterDirectorFsmStateSeparator
// ////////////////////////////////////////////////////////////////////

TtrTextWriterDirectorFsmStateSeparator &
TtrTextWriterDirectorFsmStateSeparator::instance()
{
  static TtrTextWriterDirectorFsmStateSeparator state;
  return state;
}


void 
TtrTextWriterDirectorFsmStateSeparator::enterState( TtrTextWriterDirectorFsm &fsm )
{
  if ( fsm.getCurrentLine().indentLevel() == 0 )
  {
    TTRSTATE( Paragraph );
    return;
  }

  EString strSep = fsm.getCurrentLine();
  strSep.trimSpaces();
  fsm.getBuilder().buildSeparator( strSep );
}


void 
TtrTextWriterDirectorFsmStateSeparator::exitState( TtrTextWriterDirectorFsm &fsm )
{
}


void 
TtrTextWriterDirectorFsmStateSeparator::lineIsBlank( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( BlankBlock );
}


void 
TtrTextWriterDirectorFsmStateSeparator::lineIsIndented( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateSeparator::lineIsGlued( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateSeparator::lineIsOriginal( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Paragraph );
}


void 
TtrTextWriterDirectorFsmStateSeparator::lineIsSeparator( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Separator );
}


void 
TtrTextWriterDirectorFsmStateSeparator::endOfText( TtrTextWriterDirectorFsm &fsm )
{
  TTRSTATE( Initial );
}

