// TtrUtFormatter.cpp: implementation of the TtrUtFormatter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TtrUtFormatter.h"
#include "TtrGlobalSetting.h"
#include "TtrTextRaw.h"
#include "TtrTextFormaterFactory.h"
#include "TtrTextFormater.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* Notes:
 * - should separate test for grouping: ( is connect to [
 */


TFWKREG_START( TtrUtFormatter, TfwkUnitTest )
  TFWKREG( frmtIndentBlankLine )
  TFWKREG( frmtIndentIndentedLine )
  TFWKREG( frmtIndentPreviousLineIsBlankLine )
  TFWKREG( frmtIndentLineContainsDoubleDot )
  TFWKREG( frmtIndentLineBeginWithSquareBracket )
  TFWKREG( frmtIndentLineBeginWithRoundBracket )
  TFWKREG( frmtIndentSeparatorLine )
  TFWKREG( frmtEosMode )
  TFWKREG( frmtEosModeGrouping )
  TFWKREG( frmtEosModeGroupingDoubleQuote )
  TFWKREG( frmtEosModeGroupingString )
  TFWKREG( frmtLeftMargin )
  TFWKREG( frmtRightMargin )
TFWKREG_END();

//////////////////////////////////////////////////////////////////////

TtrUtFormatter::TtrUtFormatter()
{
  _pText = new TtrTextRaw();
}

TtrUtFormatter::~TtrUtFormatter()
{
  delete _pText;
}


void 
TtrUtFormatter::frmtIndentBlankLine()
{
  static EString strText( 
      "[...]\n"   //0
      "\n"        //1
      "With a scream and a leap Sailor Sun launched herself to where\n" 
      "Uranus was guarding Saturn and Neptune. Sticking her arms in the\n" 
      "air, tried to ignore the way she could feel the bones grate in her left\n" 
      "arm. \"Sailors, join on me!\"\n"
      "\n"    //6
      "Despite not knowing what she planned, all the Sailors moved.\n" 
      "Teamwork was the key to their success. If Sailor Sun knew something\n" 
      "they did not, they did not have time to question her. Sun took one last\n" 
      "look around her, making sure her friends were close enough. Closing\n"
      "her eyes, she called on the one other attack she knew she had.\n" 
      "\"SOLAR FLARE!\"\n"
      "\n"  //13
      "A sudden concussion knocked most of the Senshi off their feet, and\n" 
      "shook the youma, causing them to pause. Even though Moon fell\n" 
      "straight onto Sailor Sun, the red haired girl did not move in the\n" 
      "slightest. While she channelled massive quantities of power, the\n" 
      "ground began to shake, and an ominous thunder rose from all around.\n"
      "The youma had just rejoined the attack when the full force appeared.\n"  //19
      );

  resetMode();
  _mode.setFormatIndentBlankLine( true );
  setText( strText );
  analyseText();

  tassert( line( 0 ).indentLevel() > 0 );
  tassert( line( 1 ).indentLevel() > 0 );
  tassert( line( 2 ).indentLevel() < 0 );
  tassert( line( 6 ).indentLevel() > 0 );
  tassert( line( 13 ).indentLevel() > 0 );
  tassert( line( 15 ).indentLevel() < 0 );
  tassert( line( 19 ).indentLevel() < 0 );
}


void 
TtrUtFormatter::frmtIndentIndentedLine()
{
  static EString strText( 
    "From: \"Triaxm'l\" <arthurh@utah-inter.net>\n"   
    "\n"          //1
    "Maximum Polarity - Balance of Power\n"
    "A BGC Fan-fiction by Arthur Hansen (Triaxm'l) arthurh@utah-inter.net\n"
    "\n"      //4
    "       Nene, stretched and yawned, and turned back to her computer.  It was\n" //5
    "late, after   midnight. Naoko was already gone, and it was probably time for\n"
    "her to head out herself. It   had actually been a slow day, and she had a\n"
    "normal day off tomorrow. No Knight Saber   business either, yay! It should\n"
    "be heaven. A sigh escaped her lips. Even work was better. It   was nice that\n"
    "the ADP had finally got to computers on the desks of the office workers.\n" //10
    "With some of the trips that she had installed, she could get some flags to\n"
    "pop up, and she   wouldn't have to leave her desk to get some of the more\n"
    "basic information for the Knight   Sabers.\n"
    "       She had even put in some alarms about her, in case somebody dug in\n"
    "too deep, and   figured her out. It was something that she put in after Dr.\n" //15
    "Mirriam's attack on the ADP,   when a girl named Lisa figured out that she\n"
    "was a Knight Saber. That was too close, Nene   thought. Ah, well, grab my\n"
    "jacket, and I'm out of here. Just as her hand reached her coat,   her\n"
    "computer beeped, in a seemingly random fashion. \"Uh, oh.\" Nene said as she\n"
    "stared at   her computer across the room.\n"   //20
    "       That was probably the biggest understatement ever uttered by Nene.\n"
    "Sliding   around to look at the screen, she saw her future at the ADP\n"
    "disintegrate. \"Oh, no!\" she   moaned as she read a memo being sent to the\n"
    "front desk concerning her arrest as a mole   for the Knight Sabers. Nene\n"
    "flinched when she saw the part of the memo that warns that   she is probably\n"//25
    "armed and dangerous.  Shoot to capture. A quick glance revealed the\n"
    "officer in charge of her capture. Matio Dansk, a known Genom mole. \"I'd\n"
    "better get out of   here. Dansk is probably already on his way over!\"\n"
    "       With a flurry of keystrokes, she sent out a handful of specialized\n" //29
    "commands that   would cripple the ADP for about an hour. And more. If she\n" //30
    );
  resetMode();
  _mode.setFormatIndentIndentedLine( true );
  setText( strText );
  analyseText();

  checkPara( 0, 4);
  checkPara( 5, 13 );
  checkPara( 14, 20 );
  checkPara( 21, 28 );
  checkPara( 29, 30 );
}


void 
TtrUtFormatter::frmtLeftMargin()
{
  static EString strText( 
    "From: \"Triaxm'l\" <arthurh@utah-inter.net>\n"   
    "\n"          //1
    " Maximum Polarity - Balance of Power\n"
    " A BGC Fan-fiction by Arthur Hansen (Triaxm'l) arthurh@utah-inter.net\n"
    "\n"      //4
    "       Nene, stretched and yawned, and turned back to her computer.  It was\n" //5
    " late, after   midnight. Naoko was already gone, and it was probably time for\n"
    " her to head out herself. It   had actually been a slow day, and she had a\n"
    "normal day off tomorrow. No Knight Saber   business either, yay! It should\n"
    " be heaven. A sigh escaped her lips. Even work was better. It   was nice that\n"
    "the ADP had finally got to computers on the desks of the office workers.\n" //10
    " With some of the trips that she had installed, she could get some flags to\n"
    "pop up, and she   wouldn't have to leave her desk to get some of the more\n"
    "basic information for the Knight   Sabers.\n"
    "       She had even put in some alarms about her, in case somebody dug in\n"
    "too deep, and   figured her out. It was something that she put in after Dr.\n" //15
    "Mirriam's attack on the ADP,   when a girl named Lisa figured out that she\n"
    " was a Knight Saber. That was too close, Nene   thought. Ah, well, grab my\n"
    "jacket, and I'm out of here. Just as her hand reached her coat,   her\n"
    "computer beeped, in a seemingly random fashion. \"Uh, oh.\" Nene said as she\n"
    "stared at   her computer across the room.\n"   //20
    "       That was probably the biggest understatement ever uttered by Nene.\n"
    "Sliding   around to look at the screen, she saw her future at the ADP\n"
    "disintegrate. \"Oh, no!\" she   moaned as she read a memo being sent to the\n"
    " front desk concerning her arrest as a mole   for the Knight Sabers. Nene\n"
    "flinched when she saw the part of the memo that warns that   she is probably\n"//25
    "armed and dangerous.  Shoot to capture. A quick glance revealed the\n"
    " officer in charge of her capture. Matio Dansk, a known Genom mole. \"I'd\n"
    "better get out of   here. Dansk is probably already on his way over!\"\n"
    "       With a flurry of keystrokes, she sent out a handful of specialized\n" //29
    "commands that   would cripple the ADP for about an hour. And more. If she\n" //30
    );
  resetMode();
  _mode.setFormatIndentIndentedLine( true );
  _mode.setFormatUseLeftMargin( true );
  _mode.setFormatLeftMargin( 1 );
  setText( strText );
  analyseText();

  checkPara( 0, 4);
  checkPara( 5, 13 );
  checkPara( 14, 20 );
  checkPara( 21, 28 );
  checkPara( 29, 30 );
}


void 
TtrUtFormatter::frmtRightMargin()
{
  static EString strText( 
    "Yohko slowly picked herself up out of the dirt for the nth time in\n"    //0
    "the last ten minutes. \"This is not going well\" she thought, shaking\n"
    "leaves from her hair. The demon was currently fencing with Azusa,\n"
    "who was only just parrying its blows. With a side of swipe of it's\n"    //3
    "tail, Azusa went spinning through the air to land next to Yohko.\n"
    "\"Aahhg! I can't even make contact with it!\" she wailed.\n"
    "Yohko considered briefly, and decided this patch of forest was\n"        //6
    "working to their disadvantage. \"Time to beat a strategic retreat,\n"
    "Apprentice.\"\n"
    "\"Huh?\"\n"    //9
    "\"Run!\"\n"
    "\"Oh!\"\n"
    "Taking off at a staggering run, they got to the street before the\n"   //12
    "demon had clued in that they were running. With a howl of horrible\n"
    "glee, it took off in pursuit.\n"
    "Running down the street in their torn and dirty dresses, spear and\n"  //15
    "soul-sword in hand, the local residents of the neighbourhood who\n"
    "were out for an evening walk were rather taken aback by the pair.\n"
    "This was nothing compared to their reaction when the demon roared\n"   //18
    "by seconds later, intent on its prey.\n"
    "\"I *pant* *wheeze* can't go *pant* much further..*pant* \" Azusa\n"
    "managed to get out three blocks from the site where they first\n"      //21
    "encountered the demon.\n"
    "\"There! Up ahead! *gasp* The old Temple! We'll make our stand\n"
    "there!\" Yohko wheezed.\n"   //24
    );
  resetMode();
  _mode.setFormatUseRightMargin( true );
  _mode.setFormatRightMargin( 67 );
  setText( strText );
  analyseText();

  checkPara( 0, 5, true);
  checkPara( 6, 8, true );
  checkPara( 9, 9, true );
  checkPara( 10, 10, true );
  checkPara( 11, 11, true );
  checkPara( 12, 14, true );
  checkPara( 15, 19, true );
  checkPara( 20, 22, true );
  checkPara( 23, 24, true );
}


void 
TtrUtFormatter::frmtIndentPreviousLineIsBlankLine()
{
  static EString strText( 
    "[...]\n"     //0
    "\n"
    "With a scream and a leap Sailor Sun launched herself to where \n"    //2
    "Uranus was guarding Saturn and Neptune. Sticking her arms in the \n"
    "air, tried to ignore the way she could feel the bones grate in her left \n"
    "arm. \"Sailors, join on me!\"\n"
    "\n"    //6
    "Despite not knowing what she planned, all the Sailors moved. \n"   //7
    "Teamwork was the key to their success. If Sailor Sun knew something \n"
    "they did not, they did not have time to question her. Sun took one last \n"
    "look around her, making sure her friends were close enough. Closing \n"
    "her eyes, she called on the one other attack she knew she had. \n"
    "\"SOLAR FLARE!\"\n"    //12
    "\n"
    "A sudden concussion knocked most of the Senshi off their feet, and \n"   //14
    "shook the youma, causing them to pause. Even though Moon fell \n"
    "straight onto Sailor Sun, the red haired girl did not move in the \n"
    "slightest. While she channelled massive quantities of power, the \n"
    "ground began to shake, and an ominous thunder rose from all around. \n"
    "The youma had just rejoined the attack when the full force appeared.\n"
    "\n"    //20
    "Centred around Sailor Sun was a small area of calm in which the \n"    //21
    "Senshi and  two youma hid, too stunned to speak or move while the \n"
    "force of nature raged beside them. A third youma almost joined them, \n"
    "but it had been caught on the invisible dividing line between safety and \n"
    "destruction. With half it's body vaporised in a raging inferno, the \n"  //25
    "youma fell to the ground and began to dissolve. All the Senshi could \n"
    "see was a dome of fire, covering them, surrounding them. A muted \n"
    "roar of flames could be heard, as though far away. Inside the dome, it \n"
    "was calm and cool. After seeing what had happened to the youma \n"
    "caught in the surrounding fires, none felt the desire to try putting a \n" //30
    "hand outside.\n"
    "\n"    //32
    "Outside it was another matter. For almost two hundred meters all \n"   //33
    );
  resetMode();
  _mode.setFormatIndentPreviousLineIsBlank( true );
  setText( strText );
  analyseText();

  tassert( line( 0 ).indentLevel() > 0 );
  tassert( line( 1 ).indentLevel() < 0 );
  tassert( line( 2 ).indentLevel() > 0 );
  tassert( line( 6 ).indentLevel() < 0 );
  tassert( line( 7 ).indentLevel() > 0 );
  tassert( line( 14 ).indentLevel() > 0 );
  tassert( line( 20 ).indentLevel() < 0 );
  tassert( line( 21 ).indentLevel() > 0 );
  tassert( line( 27 ).indentLevel() < 0 );
  tassert( line( 33 ).indentLevel() > 0 );
}


void 
TtrUtFormatter::frmtIndentLineContainsDoubleDot()
{
  static EString strText( 
    "Genma:Fine then, I'll finish you off to teach you a lesson.\n"   //0
    "\n"
    "Ranma:You'll try and finish me off that is.\n"
    "\n"
    "(Genma quickly lunges at Ranma, expecting to push him into the pond. \n"
    "Instead, Ranma just sidesteps to his left and sticks his foot out, \n"   //5
    "tripping Genma into to pond facefirst.)\n"
    "\n"
    "Ranma:What a lame attack.\n"   //8
    "\n"
    "Genma:(Resurfacing)Growl!\n"
    "\n"          //11
    "(Akane, who walked in as the two fought, giggles at the sight. She is on \n"
    "the porch. Ranma turns to see who's laughing. Genma, seeing that Ranma \n"
    "is distracted, grabs him and pulls him into the pond with him. Akane \n"
    "giggles some more as Ranma-chan surfaces.)\n"    //15
    "\n"
    "Ranma-chan:Hey old man....what did you do that for?!?\n"   //17
    );
  resetMode();
  _mode.setFormatIndentContainDoubleDot( true );
  setText( strText );
  analyseText();

  tassert( line( 0 ).indentLevel() > 0 );
  tassert( line( 1 ).indentLevel() < 0 );   
  tassert( line( 2 ).indentLevel() > 0 );
  tassert( line( 7 ).indentLevel() < 0 );
  tassert( line( 8 ).indentLevel() > 0 );
  tassert( line( 10 ).indentLevel() > 0 );
  tassert( line( 11 ).indentLevel() < 0 );
  tassert( line( 12 ).indentLevel() < 0 );
  tassert( line( 16 ).indentLevel() < 0 );
  tassert( line( 17 ).indentLevel() > 0 );
}


void 
TtrUtFormatter::frmtIndentLineBeginWithSquareBracket()
{
  static EString strText( 
    "Genma:Fine then, I'll finish you off to teach you a lesson.\n"   //0
    "\n"
    "Ranma:You'll try and finish me off that is.\n"
    "\n"
    "[Genma quickly lunges at Ranma, expecting to push him into the pond. \n"
    "Instead, Ranma just sidesteps to his left and sticks his foot out, \n"   //5
    "tripping Genma into to pond facefirst.]\n"
    "\n"
    "Ranma:What a lame attack.\n"   //8
    "\n"
    "Genma:[Resurfacing]Growl!\n"
    "\n"          //11
    "[Akane, who walked in as the two fought, giggles at the sight. She is on \n"
    "the porch. Ranma turns to see who's laughing. Genma, seeing that Ranma \n"
    "is distracted, grabs him and pulls him into the pond with him. Akane \n"
    "giggles some more as Ranma-chan surfaces.]\n"    //15
    "\n"
    "Ranma-chan:Hey old man....what did you do that for?!?\n"   //17
    );
  resetMode();
  _mode.setFormatIndentBeginWith( true );
  _mode.setFormatIndentBeginWithChar( _T( "[" ) );
  setText( strText );
  analyseText();

  tassert( line( 0 ).indentLevel() > 0 );   // !!Abnormal!!
  tassert( line( 1 ).indentLevel() < 0 );
  tassert( line( 2 ).indentLevel() < 0 );
  tassert( line( 4 ).indentLevel() > 0 );
  tassert( line( 8 ).indentLevel() < 0 );
  tassert( line( 12 ).indentLevel() > 0 );
  tassert( line( 13 ).indentLevel() < 0 );
  tassert( line( 15 ).indentLevel() < 0 );
  tassert( line( 16 ).indentLevel() < 0 );
  tassert( line( 17 ).indentLevel() < 0 );
}


void 
TtrUtFormatter::frmtIndentLineBeginWithRoundBracket()
{
  static EString strText( 
    "Genma:Fine then, I'll finish you off to teach you a lesson.\n"   //0
    "\n"
    "Ranma:You'll try and finish me off that is.\n"
    "\n"
    "(Genma quickly lunges at Ranma, expecting to push him into the pond. \n"
    "Instead, Ranma just sidesteps to his left and sticks his foot out, \n"   //5
    "tripping Genma into to pond facefirst.)\n"
    "\n"
    "Ranma:What a lame attack.\n"   //8
    "\n"
    "Genma:(Resurfacing)Growl!\n"
    "\n"          //11
    "(Akane, who walked in as the two fought, giggles at the sight. She is on \n"
    "the porch. Ranma turns to see who's laughing. Genma, seeing that Ranma \n"
    "is distracted, grabs him and pulls him into the pond with him. Akane \n"
    "giggles some more as Ranma-chan surfaces.)\n"    //15
    "\n"
    "Ranma-chan:Hey old man....what did you do that for?!?\n"   //17
    );
  resetMode();
  _mode.setFormatIndentBeginWith( true );
  _mode.setFormatIndentBeginWithChar( _T( "(" ) );
  setText( strText );
  analyseText();

  tassert( line( 0 ).indentLevel() > 0 );     // !!Abnormal!!
  tassert( line( 1 ).indentLevel() < 0 );
  tassert( line( 2 ).indentLevel() < 0 );
  tassert( line( 4 ).indentLevel() > 0 );
  tassert( line( 8 ).indentLevel() < 0 );
  tassert( line( 12 ).indentLevel() > 0 );
  tassert( line( 13 ).indentLevel() < 0 );
  tassert( line( 15 ).indentLevel() < 0 );
  tassert( line( 16 ).indentLevel() < 0 );
  tassert( line( 17 ).indentLevel() < 0 );
}


void 
TtrUtFormatter::frmtIndentSeparatorLine()
{
  static EString strText( 
    "she could finish \"...is supposed to be a private ceremony! Oh, dear. She\n"   //0
    "mustn't interfere with them...\"\n"
    "\n"    //2
    "***\n" //3
    "\n"
    "The courtyard. Soun was sitting on the porch of the house, bawling his eyes\n"   //5
    "out, as usual. Nabiki was sitting next to him: \"Now, Daddy, that's enough\n"
    "out of you.\" But Ukyou could see that Nabiki wasn't as cool and composed as\n"
    "she was pretending to be. Her eyes were red and puffy, and it was only\n"
    "because her father had his head buried in his hands that he didn't notice.\n"
    "Curiously, Akane was nowhere to be seen.\n" //10
    "\n"
    "Then she looked out into the courtyard and saw them.\n"
    "\n"    //13
    "There was a small white tent next to the pond. The fabric was of light\n"
    "enough material that the silhouettes showed through: two figures, kneeling\n"
    "in front of a standing figure.\n"
    "\n"    //17
    "This was it. This was what she'd come for. To prevent this travesty of\n"
    "justice or die trying. She charged into the tent, and threw herself down,\n"
    "prostrate, at the feet of the standing individual. An individual who, as\n"  //20
    "Ukyou looked up into her face, seemed to be the avenging angel of death\n"
    "herself.\n"
    "\n"    //23
    "\"MRS. SAOTOME!!\" Ukyou screamed \"PLEASE!! NOOOOO!!!\"\n"
    "  ------------------------------------------------------------------------\n"  //25
    "\n"    //26
    "Chapter II - Kuonji for the defense\n"
    "\n"    //28
    "\"MRS. SAOTOME!! PLEASE!! NOOOOO!!!\"\n"   //29
    );

  resetMode();
  _mode.setFormatIndentSeparator( true );
  setText( strText );
  analyseText();

  tassert( line( 0 ).indentLevel() > 0 );     // !!!abnormal!!!
  tassert( line( 1 ).indentLevel() < 0 );
  tassert( line( 2 ).indentLevel() < 0 );
  tassert( line( 3 ).indentLevel() > 0 );
  tassert( line( 8 ).indentLevel() < 0 );
  tassert( line( 13 ).indentLevel() < 0 );
  tassert( line( 17 ).indentLevel() < 0 );
  tassert( line( 24 ).indentLevel() < 0 );
  tassert( line( 25 ).indentLevel() > 0 );
  tassert( line( 26 ).indentLevel() > 0 );    
  tassert( line( 27 ).indentLevel() < 0 );

  _mode.setFormatSeparatorChar( _T( "*" ) );
  setText( strText );
  analyseText();

  tassert( line( 3 ).indentLevel() > 0 );
  tassert( line( 8 ).indentLevel() < 0 );
  tassert( line( 25 ).indentLevel() < 0 );
  tassert( line( 26 ).indentLevel() < 0 );    
  tassert( line( 27 ).indentLevel() < 0 );
}


void 
TtrUtFormatter::frmtEosMode()
{
  static EString strText( 
    "Yohko slowly picked herself up out of the dirt for the nth time in\n"    //0
    "the last ten minutes. \"This is not going well\" she thought, shaking\n"
    "leaves from her hair. The demon was currently fencing with Azusa,\n"
    "who was only just parrying its blows. With a side of swipe of it's\n"    //3
    "tail, Azusa went spinning through the air to land next to Yohko.\n"
    "\"Aahhg! I can't even make contact with it!\" she wailed.\n"
    "Yohko considered briefly, and decided this patch of forest was\n"        //6
    "working to their disadvantage. \"Time to beat a strategic retreat,\n"
    "Apprentice.\"\n"
    "\"Huh?\"\n"    //9
    "\"Run!\"\n"
    "\"Oh!\"\n"
    "Taking off at a staggering run, they got to the street before the\n"   //12
    "demon had clued in that they were running. With a howl of horrible\n"
    "glee, it took off in pursuit.\n"
    "Running down the street in their torn and dirty dresses, spear and\n"  //15
    "soul-sword in hand, the local residents of the neighbourhood who\n"
    "were out for an evening walk were rather taken aback by the pair.\n"
    "This was nothing compared to their reaction when the demon roared\n"   //18
    "by seconds later, intent on its prey.\n"
    "\"I *pant* *wheeze* can't go *pant* much further..*pant* \" Azusa\n"
    "managed to get out three blocks from the site where they first\n"      //21
    "encountered the demon.\n"
    "\"There! Up ahead! *gasp* The old Temple! We'll make our stand\n"
    "there!\" Yohko wheezed.\n"   //24
    );

  resetMode();
  _mode.setFormatModeEos( true );
  setText( strText );
  analyseText();

  checkPara( 0, 4 );
  checkPara( 5, 5 );
  checkPara( 6, 8);
  checkPara( 9, 9 );
  checkPara( 10, 10 );
  checkPara( 11, 11 );
  checkPara( 12, 14 );
  checkPara( 15, 17 );
  checkPara( 18, 19 );
  checkPara( 20, 22 );
  checkPara( 23, 24 );

  setText( strText );
  _mode.setFormatModeEosChar( _T( "." ) );
  analyseText();

  checkPara( 0, 4 );
  checkPara( 5, 5 );
  checkPara( 6, 14);
  checkPara( 15, 17 );
  checkPara( 18, 19 );
  checkPara( 20, 22 );
  checkPara( 23, 24 );
}




void 
TtrUtFormatter::frmtEosModeGrouping()
{
  static EString strText( 
    "Keiku rolls his eyes and lightly slaps me on the rump.  \"Go chase the\n"    //0
    "remaining customers out, will ya?  I want to go home.\"  The wink he \n"
    "gives brings a slight warmth to my face.  What does he think he's \n"
    "doing, treating me like a teenage girl?\n"     //3
    "\n"
    "Speaking of which, the kitchen door swings open and Miyabi, my oldest,\n"    //5
    "saunters in with a tray of empty plates.  She sees the both of us and\n"
    "groans, \"Oh, come on.  How are we ever going to close if you keep \n"
    "playing games in the kitchen?\"  [Such a tongue on her.\n"     // group  [
    "I should \n"     //9
    "chastise her for her smart mouth, but what good would it do?]  She's \n"
    "entirely too much like me.  She even looks like me, except her hair is \n"
    "shorter, pulled tight in its chestnut ponytail so that it dangles down \n"   //12
    "the nape of her neck.\n"
    "\n"
    "(But ah, there are other ways putting an insolent girl in her place.\n"  //grp( 15
    "  I smile benignly. ) \"How would you know the games we're playing?  Not \n" //grp"
    "unless you and Setsuka have been trying to learn them yourselves?  \n"
    "Maybe it's time for you and me to sit down and discussed these \n"           //18
    "'games'.\"\n"    //19
    );

  resetMode();
  _mode.setFormatModeEos( true );
  _mode.setFormatUseEnclosure( true );
  _mode.getFormatEnclosureList().addEnclosure( new TtrEnclosure( "(", ")" ) );
  _mode.getFormatEnclosureList().addEnclosure( new TtrEnclosure( "[", "]" ) );
  _mode.getFormatEnclosureList().addEnclosure( new TtrEnclosure( "\"", "\"" ) );
  setText( strText );
  analyseText();

  checkPara( 0, 3 );
  checkPara( 4, 13 );
  checkPara( 14, 19 );
}


void 
TtrUtFormatter::frmtEosModeGroupingDoubleQuote()
{
  static EString strText( 
    "Keiku rolls his eyes and lightly slaps me on the rump.  \"Go chase the\n"    //0
    "remaining customers out, will ya?  I want to go home.\"  The wink he \n"
    "gives brings a slight warmth to my face.  What does he think he's \n"
    "doing, treating me like a teenage girl?\n"     //3
    "\n"
    "Speaking of which, the kitchen door swings open and Miyabi, my oldest,\n"    //5
    "saunters in with a tray of empty plates.  She sees the both of us and\n"
    "groans, \"Oh, come on.  How are we ever going to close if you keep \n"
    "playing games in the kitchen?\"  [Such a tongue on her.\n"     // group  [
    "I should \n"     //9
    "chastise her for her smart mouth, but what good would it do?]  She's \n"
    "entirely too much like me.  She even looks like me, except her hair is \n"
    "shorter, pulled tight in its chestnut ponytail so that it dangles down \n"   //12
    "the nape of her neck.\n"
    "\n"
    "(But ah, there are other ways putting an insolent girl in her place.\n"  //grp( 15
    "  I smile benignly. ) \"How would you know the games we're playing?  Not \n" //grp"
    "unless you and Setsuka have been trying to learn them yourselves?  \n"
    "Maybe it's time for you and me to sit down and discussed these \n"           //18
    "'games'.\"\n"    //19
    );

  resetMode();
  _mode.setFormatModeEos( true );
  _mode.setFormatUseEnclosure( true );
  _mode.getFormatEnclosureList().addEnclosure( new TtrEnclosure( "\"", "\"" ) );
  setText( strText );
  analyseText();

  checkPara( 0, 3 );
  checkPara( 4, 8 );
  checkPara( 9, 13 );
  checkPara( 14, 15 );
  checkPara( 16, 19 );
}


void 
TtrUtFormatter::frmtEosModeGroupingString()
{
  static EString strText( 
    "Kyusaku  looked astonished. Well,  he did  after he translated  the slurred\n" //0
    "speech in  his head. I mean, if he couldn't  understand what the hell Genma\n"
    "was saying,  why would he be astonished initially?  What would be the point\n"
    "of  writing   this  part  if  Kyusaku   didn't  have  a  little  difficulty\n" //3
    "comprehending  the language  of your  common drunk?  There would  have been\n"
    "little point to this if he managed to unravel what it was Genma was saying,\n"
    "and--\n" //6
    "\n"
    "<<< ""Do  you think you can continue with  the story?"" asked Ranma-Neko, who\n"
    "was busy munching on a fishstick behind the author.\n"  //9
    "\n"
    """Why?"" she asked, her face still glued to the monitor.\n"
    "\n"  //12
    """I don't think the  people reading this would appreciate it if you continue\n"
    "to ramble on like that.""\n"
    "\n"  //15
    """Right. Okay,  but what the heck should I do  now that I've started writing\n"
    "that useless paragraph?""\n"
    "\n"  //18
    """Do what  everyone else  does. Continue on  with the story and  hope no one\n"
    "notices."" >>>\n"
    "\n"  //21
    """Is there  more?"" Kyusaku  inquired. That couldn't  be the only  reason, he\n"
    "thought.   Genma   nodded,  or   at   least   tried  to   lift  his   head.\n" //23
    );

  resetMode();
  _mode.setFormatIndentPreviousLineIsBlank( true );
  _mode.setFormatUseEnclosure( true );
  _mode.getFormatEnclosureList().addEnclosure( new TtrEnclosure( "<<<", ">>>" ) );
  setText( strText );
  analyseText();

  checkPara( 0, 7 );
  checkPara( 8, 21 );
  checkPara( 22, 23 );
}


void 
TtrUtFormatter::resetMode()
{
  _mode.reset();
  _mode.setFormatModeEosChar( _T(".?!\"#") );           //
  _mode.setFormatSeparatorChar( _T( "~#{([-|/_\\^@)]}=+$%*.:!,;o" ) );   //
}


void 
TtrUtFormatter::analyseText()
{
  TtrTextFormaterFactory factory;
  TtrTextFormater *pFormater = factory.createFormater( _mode );
  pFormater->formatBlock( TtrTextBlock( *_pText,
                                        0, 
                                        _pText->nbLine()-1 ),
                          _mode,
                          NULL );
  delete pFormater;
}


void
TtrUtFormatter::setText( const EString &strText )
{
  _pText->clear();
  SL nPos = 0;
  do
  {
    SL nNextPos = strText.pos( nPos, '\n' );
    SL nBreak;
    nBreak = ( nNextPos == -1 ) ? strText.length() : 
                                  nBreak = nNextPos;
    EString strLine( strText.mid( nPos, nBreak-nPos ) );    // don't include '\n'
    _pText->insertLine( new TtrTextLine( strLine ) );

    if ( nNextPos == -1 )
      break;

    nPos = nNextPos+1;    // skip the '\n'
  }
  while ( nPos < strText.length() );
}


TtrTextLine &
TtrUtFormatter::line( int nLineNo )
{
  tassert( _pText->isValidLineNo( nLineNo ) );
  return _pText->line( nLineNo );
}


void 
TtrUtFormatter::checkPara( int nFirstLine, int nLastLine, bool bDump )
{
  if ( bDump )
  {
    TRACE( "Para: %d - %d = %c %s\n", 
           nFirstLine, 
           nLastLine, 
           line(nFirstLine).indentLevel()>0 ? '>'  :  '<', 
           (const char *)line(nFirstLine) );
  }
  tassert( line(nFirstLine++).indentLevel() > 0 );
  while ( nFirstLine <= nLastLine )
  {
    if ( bDump )
    {
      TRACE( "Line: %d = %c %s\n", 
             nFirstLine, 
             line(nFirstLine).indentLevel()>0 ? '>'  :  '<', 
             (const char *)line(nFirstLine) );
    }
    tassert( line(nFirstLine++).indentLevel() < 0 );
  }
}
