/*****************************************************************************
 *
 * Autor:			Joachim Raidl
 * Klasse:			XPaneText
 * JRRC:			%J%
 * Beschreibung:	Dieses Klasse ist vollstndig inline. Mit ihr kann ein
 *					Pane der Statuszeile temporr auf Text umgeschaltet
 *					werden. Der Text bleibt solange im Pane sichtbar, bis
 *					das erzeugte XPaneText-Objekt wieder zerstrt wird. Dies
 *					erfolgt normalerweise automatisch am Ende eines Blocks.
 *					Hierbei ist es egal, ob das gewnschte Pane im Text-
 *					Bitmap-, Nummer- oder Progress-Bar-Modus erzeugt wurde.
 *					Auch ist es egal, ob der Pane gerade aktiv oder disabled
 *					ist, die Darstellung entspricht immer dem aktiven Zustand.
 * Inlines:			XPaneText(), ~XPaneText(), 
 * Beispiel:		XPaneText t("Jetzt wird gerade berechnet ...");
 *
 ****************************************************************************/

#ifndef __XPANETEXT__
#define __XPANETEXT__

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "XStatusBar.h"

class XPaneText
{
protected:
	int	index;

public:
	XPaneText(LPCSTR text, int ix = 0)
	{
		ASSERT(ms_pStatusBar);
    m_pStatusBarUsed = ms_pStatusBar;

		index = ix;
		ms_pStatusBar->SavePane(ix);
		ms_pStatusBar->SetMode(ix, XSB_TEXT | ms_pStatusBar->GetMode(ix) & XSB_ALIGN);

		COLORREF fg = ms_pStatusBar->GetFgColor(ix);
		COLORREF bk = ms_pStatusBar->GetBkColor(ix);
		ms_pStatusBar->SetFgColor(ix, fg, fg);		// Gleiche Fg- und Bk-Farbe fr
		ms_pStatusBar->SetBkColor(ix, bk, bk);		// On- und Off-Modus

		ms_pStatusBar->SetText(ix, text, text);	// Gleicher Text in beiden Modi
	}

  XPaneText( XStatusBar &statusBar, LPCSTR text, int ix = 0)
	{
		ASSERT(&statusBar);
    m_pStatusBarUsed = &statusBar;

		index = ix;
		statusBar.SavePane(ix);
		statusBar.SetMode(ix, XSB_TEXT | statusBar.GetMode(ix) & XSB_ALIGN);

		COLORREF fg = statusBar.GetFgColor(ix);
		COLORREF bk = statusBar.GetBkColor(ix);
		statusBar.SetFgColor(ix, fg, fg);		// Gleiche Fg- und Bk-Farbe fr
		statusBar.SetBkColor(ix, bk, bk);		// On- und Off-Modus

		statusBar.SetText(ix, text, text);	// Gleicher Text in beiden Modi
	}

	~XPaneText()
	{
		ASSERT(m_pStatusBarUsed);

		m_pStatusBarUsed->RestorePane(index);
	}

  static XStatusBar *GetDefaultStatusBar() { return ms_pStatusBar; }
  static void SetDefaultStatusBar( XStatusBar *pStatusBar ) { ms_pStatusBar = pStatusBar; }

private:
  XStatusBar *m_pStatusBarUsed;
  static XStatusBar *XPaneText::ms_pStatusBar;
};


#endif // __XPANETEXT__
