// ttrDoc.cpp : implementation of the CTtrDoc class
//
/* HST:
 * 29/10/97: - add glueLines & indentLines methods.
 *           - show() change hightlighted text color to black (more readable).
 *           - add version number stuff for configuration file.
 * 09/11/97: - remove bug on indentLines & glueLines when markTop =0 and
 *             markEnd<0 (the line counter was unsigned and thus the loop
 *             was going nearly infinitly).
 *           - ADD FUNCTIONNALITY:
 *             when no block define for block operation, operates on current
 *             line. Change for: unsetLines(), glueLines() and indentLines().
 *           - corrected bug on start/end block line number display: add 1.
 *           - add mode indentSquareBracket & quoteSquareBracket to support
 *             quotation of square bracket enclosed text (decribtion).
 *           - add mode indentScriptLine to indent line that begin with ':'.
 *           - add mode indentSeparator to indent that contains only one of
 *             the following char: "*+-= @".
 *             When such a line is detected then the line and the next one are
 *             indented.
 *           - correct BUG in analysis: stop at markEnd-1 instead of markEnd
 *             => last line was never analysed!
 * 20/01/98: - BUG: - analyseBlock(): lineIsSeparator not initialized to FALSE!
 *                  - First line Initialization does not take in account the 
 *                    indentSeparator mode for previousIsSeparator!
 *           - Integration de EText dans CTtrDoc
 * 23/11/98: - Add Find text.
 * 24/11/98: - Add progress while loading.
 */


/* TODO:
 */


// Note: les types de types sont stocke dans la string ADR_TTRTYPE.
// La premiere chaine correspond au nom par defaut du document.

#include "stdafx.h"
#include "resource.h"

#include "ttrDoc.h"
#include "ttrView.h"
#include "MainFrm.h"
#include "EwgFindTextInfo.h"
#include "TtrTextReaderFactory.h"
#include "TtrTextReader.h"
#include "TtrTextWriterFactory.h"
#include "TtrTextWriter.h"
#include "TtrGlobalSettingAccess.h"
#include "EwgClipboardManager.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTtrDoc

IMPLEMENT_DYNCREATE(CTtrDoc, CDocument)

BEGIN_MESSAGE_MAP(CTtrDoc, CDocument)
	//{{AFX_MSG_MAP(CTtrDoc)
	ON_COMMAND(ID_TTR_FILE_SAVE_AS_HTML, OnTtrFileSaveAsHtml)
	ON_COMMAND(ID_TTR_FILE_SAVE_AS_HTML_AS, OnTtrFileSaveAsHtmlAs)
	ON_COMMAND(ID_TTR_SAVE_TXT_TO_CLIPBOARD, OnTtrSaveTxtToClipboard)
	ON_COMMAND(ID_TTR_SAVE_HTML_TO_CLIPBOARD, OnTtrSaveHtmlToClipboard)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTtrDoc construction/destruction

CTtrDoc::CTtrDoc()
{
  reset();
}


CTtrDoc::~CTtrDoc()
{
}


BOOL 
CTtrDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

  reset();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CTtrDoc diagnostics

#ifdef _DEBUG
void CTtrDoc::AssertValid() const
{
	CDocument::AssertValid();
}


void 
CTtrDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


BOOL 
CTtrDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
  SetModifiedFlag( FALSE );

  CMainFrame *pMainFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
  if ( pMainFrame )
    pMainFrame->setTask( IDS_TTR_LOADING );

  setText( NULL );

  ETRY
  {
    TtrTextReaderFactory factory;
    EAutoPtrT<TtrTextReader> ptrReader = factory.createTextReader( lpszPathName );

    EFile file( lpszPathName );
    file.open( EIO_ReadOnly );
    EDataStream sText( &file );

    EAutoPtrT<TtrText> ptrText( ptrReader->readText( sText, *pMainFrame ) );
    text().setName( lpszPathName );
    setText( ptrText.detach() );
  }
  ECATCH_ANY
  {
    AfxMessageBox( _T( "Failed to load document!" ) );
    return FALSE;
  }

	return TRUE;
}


BOOL 
CTtrDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
  CMainFrame *pMainFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
  if ( pMainFrame )
    pMainFrame->setTask( IDS_TTR_SAVING );

  ETRY
  {
    TtrTextWriterFactory factory;
    TtrTextWriter *pWriter = 
        factory.createTextWriter( TtrGlobalSettingAccess::getGlobalSetting(),
                                  EString( lpszPathName ) );

    ETRY
    {
      EFile file( lpszPathName );
      file.open( EIO_WriteOnly );
      pWriter->writeText( TtrGlobalSettingAccess::getGlobalSetting(), 
                          TtrGlobalSettingAccess::getGlobalSetting(), 
                          text(), 
                          file, 
                          *pMainFrame );
    }
    ECATCH_ANY
    {
      delete pWriter;
      ETHROW_AGAIN;
    }
    delete pWriter;
  }
  ECATCH_ANY
  {
    AfxMessageBox( _T( "Failed to save document!") );
    return FALSE;
  }

  SetModifiedFlag( FALSE );
  
  return TRUE;
}



///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//              Methods comming from EText
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/** save text to a file
 @param fileName name of destination file
 @return Error code: ERR_FILE_OPEN
 */
SL CTtrDoc::save( const char *fileName, const TtrGlobalSetting& mode)
{
/*
  FILE *f;
  bool first = TRUE;
  SL no;

  f = fopen(fileName, "wt");
  if (!f)
    return -1;

  for (no = 0; no < nbLine; ++no)
  {
    ETextLine *line = text.at(no);

    line->save(f, mode.m_savingMode, mode.m_nReindentSize, first);

    first = FALSE;
  }
  fputc('\n', f);
  fclose(f);
  return 0;
  */
  return -1;
}


/// stick this line to the previous and remove tailing chars of previous line
void CTtrDoc::stickLineToPrevious( int nLineNo )
{
  if ( text().isValidLineNo( nLineNo )  &&
       nLineNo > 0 )
  {
    TtrTextLine &linePrev = text().line( nLineNo -1 );
    TtrTextLine &line = text().line( nLineNo );
    line.trimLeft();
    linePrev.trimRight();
    linePrev.append( line );
    text().deleteLine( nLineNo );
    SetModifiedFlag();
  }
// !!!!!!!!!!!!!!!! MUST Send a message !!!!!!!!!!!!!!!!!!!!
// line xxx deleted...      => update curline & block start & end
}


// reset
void CTtrDoc::reset()
{
  setText( new TtrTextRaw() );
}


bool 
CTtrDoc::shiftLineLeft( SL nLineNo )
{
  if ( text().isValidLineNo( nLineNo ) )
  {
    TtrTextLine &line = text().line( nLineNo );
    if ( line.indentLevel() >= 0 )
    {
      line.decIndentLevel();
      SetModifiedFlag();
      return true;
    }
  }

  return false;
}


bool 
CTtrDoc::shiftLineRight( SL nLineNo )
{
  if ( text().isValidLineNo( nLineNo ) )
  {
    TtrTextLine &line = text().line( nLineNo );
    line.incIndentLevel();
    SetModifiedFlag();
    return false;
  }  

  return true;
}


void 
CTtrDoc::textModified( bool bIsModified )
{
  SetModifiedFlag( bIsModified ? TRUE : FALSE );
}



void 
CTtrDoc::OnTtrFileSaveAsHtml() 
{
  CString strOldFilename = GetPathName();
  EFilename filename( EString( (LPCTSTR)strOldFilename ) );

  if ( !filename.filename().isNull() )
  {
    filename.setExt( "html" );
    OnSaveDocument( (LPCTSTR)filename.filename() );
  }
  else
  {
    OnTtrFileSaveAsHtmlAs();
  }
}


void 
CTtrDoc::OnTtrFileSaveAsHtmlAs() 
{
  CString strOldFilename = GetPathName();
  EFilename filename( EString( (LPCTSTR)strOldFilename ) );

  CFileDialog dlg( FALSE,
                   _T( "html" ),
                   (LPCTSTR)filename.filename(),
                   OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                   _T( "HTML Files (*.htm;*.html)|*.htm;*.html||" ) );
  if ( dlg.DoModal() == IDOK )
  {
    OnSaveDocument( dlg.GetPathName() );
  }
}


void 
CTtrDoc::setText( TtrText *pText )
{
  _ptrText = pText;

  if ( !_ptrText )
    return;
  
  _ptrText->setModifiedTarget( this );
  UpdateAllViews( NULL, ttrViewHint_textContent, NULL );
}


void 
CTtrDoc::OnTtrSaveTxtToClipboard() 
{
  saveToClipboard( text(), TtrTextWriterFactory::ttwfText );
}


void 
CTtrDoc::OnTtrSaveHtmlToClipboard() 
{
  saveToClipboard( text(), TtrTextWriterFactory::ttwfHtml );
}


void 
CTtrDoc::saveToClipboard( TtrText &text,
                          TtrTextWriterFactory::WriterFormat ttwfFormat )
{
  CMainFrame *pMainFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
  if ( pMainFrame )
    pMainFrame->setTask( IDS_TTR_SAVING );

  ETRY
  {
    TtrTextWriterFactory factory;
    EAutoPtrT<TtrTextWriter> ptrWriter( 
        factory.createTextWriter( TtrGlobalSettingAccess::getGlobalSetting(),
                                  text.getName(),
                                  ttwfFormat ) );

    EBuffer buffer;
    buffer.open( EIO_ReadWrite );
    ptrWriter->writeText( TtrGlobalSettingAccess::getGlobalSetting(), 
                          TtrGlobalSettingAccess::getGlobalSetting(), 
                          text, 
                          buffer, 
                          *pMainFrame );

    EwgClipboardManager clipboard;
    buffer.at( 0 );
    if ( !clipboard.writeTextToClipboard( buffer ) )
    {
      AfxMessageBox( _T( "Failed to copy reformatted document to clipboard!") );
      return;
    }
  
  }
  ECATCH_ANY
  {
    AfxMessageBox( _T( "Failed to copy reformatted document to clipboard!") );
    return;
  }

  SetModifiedFlag( FALSE );
}


void 
CTtrDoc::autoDetectSetting()
{
}
