// ttrDoc.h : interface of the CTtrDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRDOC_H__1B7048CC_89DC_11D1_91BC_981384E67461__INCLUDED_)
#define AFX_TTRDOC_H__1B7048CC_89DC_11D1_91BC_981384E67461__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TtrGlobalSetting.h"
#include "EwgProgressInfo.h"
#include "TtrTextRaw.h"
#include "TtrTextModifiedTarget.h"
#include "TtrTextWriterFactory.h"

class CTtrView;
class EwgFindTextInfo;


/*!
 * View Update hints.
 */
enum TtrViewHint
{
  ttrViewHint_all = 0,
  ttrViewHint_textContent,   // lines have been added or deleted or changed
  ttrViewHint_textNew,       // The text has been completly changed.
};


class CTtrDoc : public CDocument,
                public TtrTextModifiedTarget
{
protected: // create from serialization only
	CTtrDoc();
	DECLARE_DYNCREATE(CTtrDoc)

// Attributes

// Operations

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTtrDoc)
	public:
		virtual BOOL OnNewDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
	virtual ~CTtrDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
	//{{AFX_MSG(CTtrDoc)
	afx_msg void OnTtrFileSaveAsHtml();
	afx_msg void OnTtrFileSaveAsHtmlAs();
	afx_msg void OnTtrSaveTxtToClipboard();
	afx_msg void OnTtrSaveHtmlToClipboard();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

public:

  SL save( const char *filename, const TtrGlobalSetting& mode );

  // shift a line attribute. 
  // Return TRUE if the line attribute is modified.
  bool shiftLineRight( SL nLineNo );
  // shift a line attribute. 
  // Return TRUE if the line attribute is modified.
	bool shiftLineLeft( SL nLineNo );

  /// stick this line to the previous and remove tailing chars of previous line
  void stickLineToPrevious( int nLineNo );

  /** Get the text that is attached to the document.
   */
  TtrText &text();

  /** Called when the text is modified.
   *
   * @param bIsModified \c true if the text has been modified.
   */
  virtual void textModified( bool bIsModified );

  /*! Specify the text that is associated to the document.
   * @param pText Text that is now associated with the document (and owned).
   */
  void setText( TtrText *pText );

private:
    void reset();

    void saveToClipboard( TtrText &text,
                          TtrTextWriterFactory::WriterFormat ttwfFormat );

    void autoDetectSetting();


private:
  EAutoPtrT<TtrText> _ptrText;
};

/////////////////////////////////////////////////////////////////////////////


// Inlines:
// Last change: 10/04/97 23h15mn


inline TtrText &
CTtrDoc::text()
{
  if ( !_ptrText )
    _ptrText = new TtrTextRaw;
  return *_ptrText;
}


    
//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TTRDOC_H__1B7048CC_89DC_11D1_91BC_981384E67461__INCLUDED_)
