// ttrView.h : interface of the CTtrView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRVIEW_H__1B7048CE_89DC_11D1_91BC_981384E67461__INCLUDED_)
#define AFX_TTRVIEW_H__1B7048CE_89DC_11D1_91BC_981384E67461__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <afxtempl.h>
#include "EBitmap.h"
#include "ChildFrm.h"
#include "EwgScrollView.h"
#include "TtrGlobalSettingListener.h"
#include "ttrDoc.h"

class CTtrDoc;
class EwgFindTextInfo;
class TtrFormatSetting;

class CTtrView : public EwgScrollView,
                 public TtrGlobalSettingListener  
{
protected: // create from serialization only
	CTtrView();
	DECLARE_DYNCREATE(CTtrView)

// Attributes
	CTtrDoc* GetDocument() const;

  // action possible
  enum Action
  {
    actionNothing =0,
    actionBlockIndent,
    actionBlockGlue,
    actionBlockNormal,
    actionLineIndent,
    actionLineGlue,
    actionLineNormal,
    actionBlockSetStart,
    actionBlockSetEnd,
    actionLineSetCurrent,
    actionBlockSelectAll,
    actionBlockSelectNone,
  };

  /*!
   * Called when the global setting changed.
   * @param setting Settting that changed.
   */
  virtual void globalSettingChanged( TtrGlobalSetting &setting );

// Operations

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTtrView)
		virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
  virtual void OnUpdate( CView* pSender, LPARAM lHint, CObject* pHint );
	//}}AFX_VIRTUAL

// Implementation
public:

  void DoFindText( const EwgFindTextInfo &info );
	void DoSelectTo( int nLineNo );
	void DoSelectUp( int step );
	void DoSelectDown( int step );
	CChildFrame * GetParentFrame() const;
  // Convert point coordinate to Logical 
	CPoint DPtoLP( CPoint& point );
	void RedrawCurrentLine();
	void RedrawBlock();
	void DoAction( Action action, int nLineNo =-1 );
	void CheckBlock();
	void HideBlock();
	void ChangeCurrentLineTo( int lineNo );
	void CenterCurrentLine();
	void ChangeCurrentLine( int move );
	void UpdateDocSize();
	CSize GetDocSize() const;
	virtual ~CTtrView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
	//{{AFX_MSG(CTtrView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnBlockIndent();
	afx_msg void OnBlockGlue();
	afx_msg void OnBlockNormal();
	afx_msg void OnLineIndent();
	afx_msg void OnLineGlue();
	afx_msg void OnLineNormal();
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnBlockAnalyse();
	afx_msg void OnUpdateBlockAnalyse(CCmdUI* pCmdUI);
	afx_msg void OnUpdateBlockGlue(CCmdUI* pCmdUI);
	afx_msg void OnUpdateBlockNormal(CCmdUI* pCmdUI);
	afx_msg void OnUpdateBlockIndent(CCmdUI* pCmdUI);
	afx_msg void OnPuBlockSetStart();
	afx_msg void OnPuBlockSetEnd();
	afx_msg void OnBlockSelectAll();
	afx_msg void OnBlockSelectNone();
	afx_msg void OnUpdateBlockSelectAll(CCmdUI* pCmdUI);
	afx_msg void OnUpdateBlockSelectNone(CCmdUI* pCmdUI);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnTtrSearchText();
	afx_msg void OnUpdateBlockAutoSetRightMargin(CCmdUI* pCmdUI);
	afx_msg void OnBlockAutoSetRightMargin();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP();
private:
/** @supplierCardinality 1*/
  CTtrDoc * pCTtrDoc;
	void BlockArrangeOrder();

  // return TRUE if the block is valid (defined).
  BOOL BlockIsValid() const;

  void StopAction();

  // Return the line at the action position 'm_actionPos'.
	int GetActionLine() const;

  // Return the line at the y logival coordinate 'yLog'.
	int LineAt( int yLog ) const;

  void RedrawLine( int lineNo );

  void UpdateScrollSizes();

  CSize GetLineSize() const;

  CSize GetPageSize() const;

  TtrFormatSetting &getFormatSetting();


	HACCEL m_hAccelerator;
	CEBitmap m_bmBlockEnd;
	CEBitmap m_bmBlockStart;
	CMenu m_popup;
  // button that started the action
  UINT m_actionBtn;
  // position of the mouse when the button was pressed down
  CPoint m_actionPos;
  // current action
  Action m_action;

  int m_blockEndNo;
	int m_blockStartNo;
	int m_curLineNo;
	TEXTMETRIC m_fontTM;
	int m_docHeight;
	int m_maxLineWidth;
	int m_lineHeight;
  CEBitmap m_bmStartPara;
  CEBitmap m_bmGluedLine;
  CMap<Action,Action,HCURSOR,HCURSOR&> m_mapCursors;
  int m_nMarginWidth;
  bool _bAnalizingText;
};

#ifndef _DEBUG  // debug version in ttrView.cpp
inline CTtrDoc* CTtrView::GetDocument() const
{ 
  return (CTtrDoc*)m_pDocument; 
}

inline CChildFrame * CTtrView::GetParentFrame() const 
{
  return (CChildFrame *)EwgScrollView::GetParentFrame();
}

#endif

inline BOOL CTtrView::BlockIsValid() const
{
  return m_blockStartNo <= m_blockEndNo;
}


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TTRVIEW_H__1B7048CE_89DC_11D1_91BC_981384E67461__INCLUDED_)
